;+
;   name:
;	rob_mosaic.pro
;
;   purpose:
;	This function(and associated GUI) allows the selection
;       of a sub-region of master, and if necessary, move the 
;	corresponding sub-region selected in slave to facilitate
;	mosaic creation.  The selections are then tracked using
;	xyoff.pro. 
;	
;
;   calling sequence:
;	result = rob_mosaic(master,slave,redo=redo,test=test)
;
;   inputs:
;	master = reference image
;	slave = image to track with respect to master
;
;   keywords:
;       redo = set to create REDO PREVIOUS button (used with assemble.pro)
;       test = set to create KILL IT button to kill process during testing
;
;   output:
;       2-element vector containing x and y offsets between master 
;	and slave to feed to mosaic_two.pro
;
;   To use rob_mosaic.pro: 
;       1) use the slider bars to scale the images displayed; 
;       2) click and drag on the master image to select the region to track on;
;       3) if necessary, click and drag the corresponding box on the slave
;              image to select the region that matches the one selected
;              in the master;
;       4) hit the SUBMIT button to make enter your selection and allow
;              the computer to determine the offset;
;       5) if using through assemble.pro, hit the REDO PREVIOUS button to 
;              jump back to the previous master and slave to re-register them
;              (can only go back one image!!)
;
;   called programs:
;
;   history:
;       initial coding 6-7-01: R. Gutermuth
;       completed     6-12-01: R. Gutermuth
;       6-18-01: fixed box redrawing bug that occured on image rescale (RAG)
;	6-20-01: added pixel value label (RAG)
;
;-



pro rob_mosaic_event,ev

widget_control,ev.top,get_uvalue=rmuv,/no_copy

case ev.id of

 rmuv.mmin: begin
  widget_control,ev.id,get_value=in
  widget_control,rmuv.mmax,get_value=mx
  if in ge mx then rmuv.mscr = byte(rmuv.master*0) else begin
   rmuv.mscr = bytscl(rmuv.master,min=in,max=mx)
  endelse
  wset,rmuv.mwn
  tv,rmuv.mscr
  if abs(rmuv.ds[0,1]-rmuv.ds[0,2]) gt 5 and $
     abs(rmuv.ds[1,1]-rmuv.ds[1,2]) gt 5 then begin
   plots,[rmuv.ds[0,1],rmuv.ds[0,1],rmuv.ds[0,2],rmuv.ds[0,2],rmuv.ds[0,1]],$
      [rmuv.ds[1,1],rmuv.ds[1,2],rmuv.ds[1,2],rmuv.ds[1,1],rmuv.ds[1,1]],$
      /device
  endif
 end

 rmuv.mmax: begin
  widget_control,ev.id,get_value=in
  widget_control,rmuv.mmin,get_value=mn
  if in le mn then rmuv.mscr = byte(rmuv.master*0) else begin
   rmuv.mscr = bytscl(rmuv.master,min=mn,max=in)
  endelse
  wset,rmuv.mwn
  tv,rmuv.mscr
  if abs(rmuv.ds[0,1]-rmuv.ds[0,2]) gt 5 and $
     abs(rmuv.ds[1,1]-rmuv.ds[1,2]) gt 5 then begin
   plots,[rmuv.ds[0,1],rmuv.ds[0,1],rmuv.ds[0,2],rmuv.ds[0,2],rmuv.ds[0,1]],$
      [rmuv.ds[1,1],rmuv.ds[1,2],rmuv.ds[1,2],rmuv.ds[1,1],rmuv.ds[1,1]],$
      /device
  endif
 end

 rmuv.smin: begin
  widget_control,ev.id,get_value=in
  widget_control,rmuv.smax,get_value=mx
  if in ge mx then rmuv.sscr = byte(rmuv.slave*0) else begin
   rmuv.sscr = bytscl(rmuv.slave,min=in,max=mx)
  endelse
  wset,rmuv.swn
  tv,rmuv.sscr
  xsh = rmuv.ds[0,5]
  ysh = rmuv.ds[1,5]
  if abs(rmuv.ds[0,1]-rmuv.ds[0,2]) gt 5 and $
     abs(rmuv.ds[1,1]-rmuv.ds[1,2]) gt 5 then begin
   plots,[rmuv.ds[0,1]+xsh,rmuv.ds[0,1]+xsh,rmuv.ds[0,2]+xsh,rmuv.ds[0,2]$
      +xsh,rmuv.ds[0,1]+xsh],[rmuv.ds[1,1]+ysh,rmuv.ds[1,2]+ysh,rmuv.ds[1,2]$
      +ysh,rmuv.ds[1,1]+ysh,rmuv.ds[1,1]+ysh],/device
  endif
 end

 rmuv.smax: begin
  widget_control,ev.id,get_value=in
  widget_control,rmuv.smin,get_value=mn
  if in le mn then rmuv.sscr = byte(rmuv.slave*0) else begin
   rmuv.sscr = bytscl(rmuv.slave,min=mn,max=in)
  endelse
  wset,rmuv.swn
  tv,rmuv.sscr
  xsh = rmuv.ds[0,5]
  ysh = rmuv.ds[1,5]
  if abs(rmuv.ds[0,1]-rmuv.ds[0,2]) gt 5 and $
     abs(rmuv.ds[1,1]-rmuv.ds[1,2]) gt 5 then begin
   plots,[rmuv.ds[0,1]+xsh,rmuv.ds[0,1]+xsh,rmuv.ds[0,2]+xsh,rmuv.ds[0,2]$
      +xsh,rmuv.ds[0,1]+xsh],[rmuv.ds[1,1]+ysh,rmuv.ds[1,2]+ysh,rmuv.ds[1,2]$
      +ysh,rmuv.ds[1,1]+ysh,rmuv.ds[1,1]+ysh],/device
  endif
 end

 rmuv.mw: begin

  case 1 of
   ev.x gt rmuv.msz[1]-1: nx = rmuv.msz[1]-1
   ev.x lt 0: nx = 0
   else: nx = ev.x
  endcase

  case 1 of
   ev.y gt rmuv.msz[2]-1: ny = rmuv.msz[2]-1
   ev.y lt 0: ny = 0
   else: ny = ev.y
  endcase

  if nx gt rmuv.ssz[1]-1 then xd=rmuv.ssz[1]-1-nx else xd=0
  if ny gt rmuv.ssz[2]-1 then yd=rmuv.ssz[2]-1-ny else yd=0

  case ev.type of

   0: begin
    if rmuv.ds[0,0] eq 0 then begin
     wset,rmuv.mwn
     tv,rmuv.mscr
    endif
    rmuv.ds[0,*] = [0,nx,0,0,0,xd]
    rmuv.ds[1,*] = [0,ny,0,0,0,yd]
   end

   1: begin
    rmuv.ds[0,*] = [-1,rmuv.ds[0,1],nx,0,0,rmuv.ds[0,5]]
    rmuv.ds[1,*] = [-1,rmuv.ds[1,1],ny,0,0,rmuv.ds[1,5]]
    if abs(rmuv.ds[0,2]-rmuv.ds[0,1]) gt 5 or $
       abs(rmuv.ds[1,2]-rmuv.ds[1,1]) gt 5 then begin
     widget_control,rmuv.ok,sensitive=1
     widget_control,rmuv.sw,sensitive=1
    endif else begin
     wset,rmuv.mwn
     tv,rmuv.mscr
     wset,rmuv.swn
     tv,rmuv.sscr
     widget_control,rmuv.ok,sensitive=0
     widget_control,rmuv.sw,sensitive=0
    endelse
   end

   2: begin
    widget_control,rmuv.xl,set_value='X: '+strtrim(nx,2)
    widget_control,rmuv.yl,set_value='Y: '+strtrim(ny,2)
    widget_control,rmuv.il,set_value='Value: '+strtrim(rmuv.master[nx,ny],2)
    if rmuv.ds[0,0] eq 0 then begin
     if abs(rmuv.ds[0,1]-nx)+1 le rmuv.ssz[1] and $
        abs(rmuv.ds[1,1]-ny)+1 le rmuv.ssz[2] then begin 
      wset,rmuv.mwn
      tv,rmuv.mscr
      plots,[rmuv.ds[0,1],rmuv.ds[0,1],nx,nx,rmuv.ds[0,1]],$
         [rmuv.ds[1,1],ny,ny,rmuv.ds[1,1],rmuv.ds[1,1]],/device
      if nx gt rmuv.ds[0,1] then rmuv.ds[0,5] = xd
      if ny gt rmuv.ds[1,1] then rmuv.ds[1,5] = yd
      wset,rmuv.swn
      tv,rmuv.sscr
      plots,[rmuv.ds[0,1]+rmuv.ds[0,5],rmuv.ds[0,1]+rmuv.ds[0,5],$
         nx+rmuv.ds[0,5],nx+rmuv.ds[0,5],rmuv.ds[0,1]+rmuv.ds[0,5]],$
         [rmuv.ds[1,1]+rmuv.ds[1,5],ny+rmuv.ds[1,5],ny+rmuv.ds[1,5],$
         rmuv.ds[1,1]+rmuv.ds[1,5],rmuv.ds[1,1]+rmuv.ds[1,5]],/device
     endif
    endif
   end

  endcase
 end

 rmuv.sw: begin

  case 1 of
   ev.x gt rmuv.ssz[1]-1: nx = rmuv.ssz[1]-1
   ev.x lt 0: nx = 0
   else: nx = ev.x
  endcase

  case 1 of
   ev.y gt rmuv.ssz[2]-1: ny = rmuv.ssz[2]-1
   ev.y lt 0: ny = 0
   else: ny = ev.y
  endcase

  x1 = min(rmuv.ds[0,1:2]) & x2 = max(rmuv.ds[0,1:2])
  y1 = min(rmuv.ds[1,1:2]) & y2 = max(rmuv.ds[1,1:2])

  if rmuv.ds[0,0] eq 0 then begin
   xsh = nx-rmuv.ds[0,3]+rmuv.ds[0,5] & ysh = ny-rmuv.ds[1,3]+rmuv.ds[1,5]

   case 1 of
    x2+xsh gt rmuv.ssz[1]-1: nx=fix(rmuv.ssz[1]-1+rmuv.ds[0,3]-rmuv.ds[0,5]-x2)
    x1+xsh lt 0: nx=fix(rmuv.ds[0,3]-rmuv.ds[0,5]-x1)
    else: nx = nx
   endcase

   case 1 of
    y2+ysh gt rmuv.ssz[2]-1: ny=fix(rmuv.ssz[2]-1+rmuv.ds[1,3]-rmuv.ds[1,5]-y2)
    y1+ysh lt 0: ny=fix(rmuv.ds[1,3]-rmuv.ds[1,5]-y1)
    else: ny = ny
   endcase

   xsh = nx-rmuv.ds[0,3]+rmuv.ds[0,5] & ysh = ny-rmuv.ds[1,3]+rmuv.ds[1,5]
  endif

  case ev.type of

   0: begin
    if rmuv.ds[0,0] eq 0 then begin
     wset,rmuv.swn
     tv,rmuv.sscr
    endif
    if (nx le x2+rmuv.ds[0,5] and nx ge x1+rmuv.ds[0,5]) and $
       (ny le y2+rmuv.ds[1,5] and ny ge y1+rmuv.ds[1,5]) then begin
     rmuv.ds[*,0] = [0,0]
     rmuv.ds[*,3] = [nx,ny] 
    endif
   end

   1: begin
    rmuv.ds[*,0] = [-1,-1]
    rmuv.ds[*,5] = rmuv.ds[*,5]+[nx,ny]-rmuv.ds[*,3]
   end

   2: begin
    widget_control,rmuv.xl,set_value='X: '+strtrim(nx,2)
    widget_control,rmuv.yl,set_value='Y: '+strtrim(ny,2)
    widget_control,rmuv.il,set_value='Value: '+strtrim(rmuv.slave[nx,ny],2)
    if rmuv.ds[0,0] eq 0 then begin
     wset,rmuv.swn
     tv,rmuv.sscr
     plots,[x1+xsh,x1+xsh,x2+xsh,x2+xsh,x1+xsh],$
           [y1+ysh,y2+ysh,y2+ysh,y1+ysh,y1+ysh],/device
    endif
   end

  endcase 

 end

 rmuv.help: begin
  dl,'rob_mosaic'
 end

 rmuv.ok: begin
  xsh = rmuv.ds[0,5]
  ysh = rmuv.ds[1,5]
  x1 = min(rmuv.ds[0,1:2]) & x2 = max(rmuv.ds[0,1:2])
  y1 = min(rmuv.ds[1,1:2]) & y2 = max(rmuv.ds[1,1:2])
  mp = rmuv.master[x1:x2,y1:y2]
  sp = rmuv.slave[x1+xsh:x2+xsh,y1+ysh:y2+ysh]
  sz = size(mp)
  ss = [x1,y1,x2,y2,rmuv.ds[0,5],rmuv.ds[1,5],1,1]

  openw,one,'anger.tmp',/get
  writeu,one,ss
  free_lun,one

  widget_control,ev.top,/destroy
  return
 end
 
 rmuv.redo: begin
  ss = fltarr(8)
  openw,one,'anger.tmp',/get
  writeu,one,ss
  free_lun,one
  widget_control,ev.top,/destroy
  return
 end
   
 rmuv.exit: begin
  widget_control,ev.top,set_uvalue=rmuv,/no_copy
  widget_control,ev.top,/destroy
  retall
 end

endcase

widget_control,ev.top,set_uvalue=rmuv,/no_copy

end


pro rob_mosaic,mastername,slavelist,mmask,slavemasks,redo=redo,test=test

fits_read,mastername,master
slnames = loadlist(slavelist) 
slave = master*0

mmo = moment(master,sdev=msd)
smo = moment(slave,sdev=ssd)
mmo = [mmo[0],msd]
smo = [smo[0],ssd]

mscr = bytscl(master,min=mmo[0]-2*mmo[1],max=mmo[0]+2*mmo[1])
sscr = bytscl(slave,min=smo[0]-2*smo[1],max=smo[0]+2*smo[1])

main = widget_base(/row,title='rob_mosaic')

bbase = widget_base(main,/column)
mbase = widget_base(main,/column)
sbase = widget_base(main,/column)

mmin = cw_fslider(mbase,/edit,min=mmo[0]-10*mmo[1],max=mmo[0]+10*mmo[1],$
      value=mmo[0]-2*mmo[1],title='Master Min',/frame)
mmax = cw_fslider(mbase,/edit,min=mmo[0]-10*mmo[1],max=mmo[0]+10*mmo[1],$
      value=mmo[0]+2*mmo[1],title='Master Max',/frame)
smin = cw_fslider(sbase,/edit,min=smo[0]-10*smo[1],max=smo[0]+10*smo[1],$
      value=smo[0]-2*smo[1],title='Slave Min',/frame)
smax = cw_fslider(sbase,/edit,min=smo[0]-10*smo[1],max=smo[0]+10*smo[1],$
      value=smo[0]+2*smo[1],title='Slave Max',/frame)

msz = size(master)
if msz[1] gt 512 then sx = 512 else sx = msz[1]
if msz[2] gt 512 then sy = 512 else sy = msz[2]
mwin = widget_draw(mbase,/scroll,/retain,/motion_events,/button_events,$
      xsize=msz[1],ysize=msz[2],x_scroll_size=sx,y_scroll_size=sy)

ssz = size(slave)
if ssz[1] gt 512 then sx = 512 else sx = ssz[1]
if ssz[2] gt 512 then sy = 512 else sy = ssz[2]
swin = widget_draw(sbase,/scroll,/retain,/motion_events,/button_events,$
      xsize=ssz[1],ysize=ssz[2],x_scroll_size=sx,y_scroll_size=sy)

help = widget_button(bbase,value='HELP')
skip = widget_button(bbase,value='SKIP',sensitive=0)
ud = widget_button(bbase,value='UNDO')
exit = widget_button(bbase,value='EXIT')

xl = widget_label(bbase,value='X: ',/dynamic_resize)
yl = widget_label(bbase,value='Y: ',/dynamic_resize)
il = widget_label(bbase,value='Value: ',/dynamic_resize)

ss = fltarr(2)
ds = fltarr(2,6)
ds[*,0] = -1

rmuv = {m:main,mb:mbase,sb:sbase,mmin:mmin,mmax:mmax,smin:smin,smax:smax,$
        mw:mwin,sw:swin,help:help,ok:ok,xl:xl,yl:yl,ss:ss,master:master,$
        slave:slave,mscr:mscr,sscr:sscr,ds:ds,mmo:mmo,smo:smo,msz:msz,$
        ssz:ssz,mwn:0l,swn:0l,exit:exit,redo:rd,il:il}

widget_control,main,/realize

widget_control,mwin,get_value=mw
rmuv.mwn=mw
wset,mw
tv,mscr

widget_control,swin,get_value=sw
rmuv.swn=sw
wset,sw
tv,sscr
widget_control,swin,sensitive=0

widget_control,main,set_uvalue=rmuv,/no_copy
xmanager,'rob_mosaic',main

end



