;+
;   name:
;        sf_dark.pro
;
;   purpose:
;       a procedure which shows a slideshow of the fits images in a given
;       directory (of dark frames, hopefully) to the user.  On user approval,
;       the darks are median_stacked and the result is returned in an
;       anonymous structure.
;
;   calling sequence:
;       result = sf_dark(channel=channel,dirname=dirname)
;
;   inputs:
;       NONE
;
;   keywords:
;       dirname = (string) subdirectory containing desired darks
;       channel = 1 character string with channel to match
;
;   output:
;       a anon structure containing the median dark image, FITS
;       header, mask vector, and date from the header.
;
;   called programs:
;       fits_datmask.pro
;       sxpar.pro
;       sxaddpar.pro
;       bytecube.pro
;       mov3r.pro
;       median_stack.pro
;
;   history:
;       written by R. Gutermuth - 5-23-01
;       8-24-01 - added multi-platform capability and changed to fits_datmask
;                 usage to use individual masks for each image.  (RAG)
;       8-29-01 - improved memory management by using pointers (now that I know
;                 how to use them...) for large arrays. (RAG)
;       1-24-02 - removed all data sorting to modularize data processing
;                 routines. Using filepath.pro for multi-platform
;                 capabilities.(RAG)
;       2-12-02 - switched to standard sxpar.pro and sxaddpar.pro
;                 usage for header info and modification (RAG)
;
;-

function sf_dark,channel=channel,dirname=dirname

;get dirname if not provided
 if not keyword_set(dirname) then dirname = dialog_pickfile(/directory,$
    title='Select a directory with dark current images')
 if dirname eq '' then begin
  print,'ERROR in sf_dark.pro: No directory specified!'
  retall
 endif

;get dark frame filenames
 f = findfile(dirname+'*?.?*',count=nf)
 if nf eq 0 then begin
  print,'ERROR in sf_dark.pro: No files of that specification!!'
  retall
 endif

;read in darks and merge their masks
 for i = 0,nf-1 do begin

  print,'Reading in '+f[i]+'...'
  fits_datmask,f[i],im,hdr,mask

  if i eq 0 then begin
   sim = size(im)

;mess around with the format of the date
   date = sxpar(hdr,'DATE-OBS')
   if rstrpos(date,'-') ne -1 then div = '-' else div = '/'
   date = strjoin(strsplit(date,div,/extract))

;boot 'em if the channel entries don't match
   if not keyword_set(channel) then channel = sxpar(hdr,'CHANNEL') else begin
    if sxpar(hdr,'CHANNEL') ne channel then begin
     print,'ERROR in sf_dark.pro: Channel entry mismatch!'
     retall
    endif
   endelse

   darks = ptr_new(make_array(sim[1],sim[2],nf,type=sim[sim[0]+1],/nozero))
   dmask = temporary(mask)
  endif else begin
   dmask = merge_mask(mask,temporary(dmask),/agree)
  endelse

  (*darks)[*,*,i] = im

 endfor

;create dir 'calib_images' if it doesn't exist
 resdir = filepath(root_dir=dirname+'..',subdirectory=['..','calib_images'],'')
 file_mkdir,resdir

;make slideshow
 dkb = bytecube(*darks,mask=dmask)

;show slideshow, get approval, then median and write final dark
 if fix(dkb[0]) eq -1 then begin
  print,'ERROR in sf_dark.pro:  Something broke in bytecube.pro!!'
  retall
 endif else begin
  window,/free,xs=sim[1]+75,ys=sim[2]+75,title="Current Dark Frames"
  mov3r,dkb
  huh = dialog_message(/question,'Median and save this dark frame?')
  wdelete,!d.window
  if huh eq 'Yes' then begin
   result = median_stack(*darks)
   sxaddpar,hdr,'HISTORY','median of '+strtrim(sim[3],2)+' images'
;   fits_datmask,/write,resdir+date+'_'+channel+'_dark.fit',result,hdr,dmask
;   print,resdir+date+'_'+channel+'_dark.fit written...'
  endif else begin
   print,'Dark Frame rejected...  Exitting'
   retall
  endelse
 endelse

;dump darks from memory
 ptr_free,darks
 stuff = {dark:result,hdr:hdr,mask:dmask,date:date}
 return,stuff

end

