
;+
; NAME:
;   PV_FIND_DETECT
;
; PURPOSE:
;   Find positive brightness perturbations (i.e stars) in an image
;       Calculation routine for GUI package photvis.pro
;
; EXPLANATION:
;   Also returns centroids and shape parameters (roundness & sharpness).
;   Adapted from 1986 STSDAS version of DAOPHOT.
;
; CALLING SEQUENCE:
;   pv_find_detect,image_ptr,info_vect,conv_ptr,result_ptr,stat_ptr[, /verbose]
;
; INPUTS:
;   image_ptr - pointer to 2 dimensional image array (integer or float) for
;               which one wishes to identify the stars present
;   info_vect - vector containing thresholds of form [fwhm,
;                   hmin,roundmin,roundmax,sharpmin,sharpmax]
;   conv_ptr - set to variable to receive pointer to convolved image or set to
;              pointer to existing convolved image to run search on
;   result_ptr- set to variable to receive pointer to array of detection data
;
; OPTIONAL INPUT KEYWORDS:
;   verbose - Normally, pv_find_detect will not write out each star that 
;             meets all selection criteria.   If the VERBOSE keyword is set and
;             non-zero, then there is a printout.
;
; OUTPUTS:
;   see conv_ptr and result_ptr above
;
; NOTES:
;   The sharpness statistic compares the central pixel to the mean of the
;   surrounding pixels.   If this difference is greater than the originally
;   estimated height of the Gaussian or less than 0.2 the height of the
;   Gaussian (for the default values of SHARPLIM) then the star will be
;   rejected.
;
; REVISION HISTORY:
;       Written for amop (now photvis) by R. Gutermuth August 2001
;         (based heavily on find.pro by W. Lansman, Feb. 1987 to Sept. 1997)
;       10-04-01 - cleaned up, made ready for use in PhotVis (RAG)
;       10-11-01 - split pv_find into pv_find_detect and pv_find_search (RAG)
;       3-26-02 - added stat_ptr (RAG)
;       3-29-02 - added centroid keyword (RAG)
;
;-

pro pv_find_detect, image_ptr, info_vect, conv_ptr, result_ptr, stat_ptr,$
             aper_ptr, verbose = verbose, centroid=centroid

;trying to keep user adds around...
 ssz = size(*stat_ptr)
 rsz = size(*result_ptr)
 asz = size(*aper_ptr)
 if ssz[0] gt 0 and rsz[0] gt 0 and asz[0] gt 0 then begin
  w = where(*stat_ptr lt 0,uct)
  if uct gt 0 then begin
   user_dat = (*result_ptr)[*,w]
   user_apers = (*aper_ptr)[*,w]
   user_stat = (*stat_ptr)[w]
  endif
 endif else uct = 0

;preserves old code
 fwhm = info_vect[0]
 hmin = info_vect[1]

 npar   = N_params()
 if npar EQ 0 then begin
    print,'Syntax - pv_find, image_ptr, info_vect, conv_ptr, result_ptr,' + $
          '[ /verbose]'
    return
 endif

 type = size(*image_ptr)
 if ( type[0] NE 2 ) then begin
  huh=dialog_message(/error,'Image array must be 2 dimensional')
  return
 endif

 maxbox = min(type[1:2]-1)<13    ;Maximum size of convolution box in pixels

 n_x  = type[1] & n_y = type[2]

 radius = 0.637*FWHM > 2.001             ;Radius is 1.5 sigma
 radsq = radius^2
 nhalf = fix(radius) < (maxbox-1)/2     ;
 nbox = 2*nhalf + 1 ;# of pixels inside of convolution box
 middle = nhalf          ;Index of central pixel

 lastro = n_x - nhalf
 lastcl = n_y - nhalf
 sigsq = ( fwhm/2.35482 )^2
 mask = bytarr( nbox, nbox )   ;Mask identifies valid pixels in convolution box
 c = fltarr( nbox, nbox )      ;c will contain Gaussian convolution kernel

 dd = indgen(nbox-1) + 0.5 - middle ;Constants need to compute ROUND
 dd2 = dd^2
 w = 1. - 0.5*(abs(dd)-0.5) / (middle-.5)
 ir = (nhalf-1) > 1

 row2 = (findgen(Nbox)-nhalf)^2

 for i = 0, nhalf do begin
    temp = row2 + i^2
    c[0,nhalf-i] = temp
        c[0,nhalf+i] = temp
 endfor

 mask = fix(c LE radsq)     ;MASK is complementary to SKIP in Stetson's Fortran
 good = where( mask, pixels)  ;Value of c are now equal to distance to center

 c = c*mask
 c[good] = exp(-0.5*c[good]/sigsq)  ;Make c into a gaussian kernel
 sumc = total(c)
 sumcsq = total(c^2) - sumc^2/pixels
 sumc = sumc/pixels
 c[good] = (c[good] - sumc)/sumcsq
 c1 = exp(-.5*row2/sigsq)
 sumc1 = total(c1)/nbox
 sumc1sq = total(c1^2) - sumc1
 c1 = (c1-sumc1)/sumc1sq
 sumc = total(w)                         ;Needed for centroid computation

 if (*conv_ptr)[0] eq -1 then begin
  h = convol(float((*image_ptr)),c)    ;Convolve image with kernel "c"
  h[0:nhalf-1,*] = 0 & h[n_x-nhalf:n_x-1,*] = 0
  h[*,0:nhalf-1] = 0 & h[*,n_y-nhalf:n_y-1] = 0
 endif else h = temporary(*conv_ptr)

 mask[middle,middle] = 0   ;From now on we exclude the central pixel
 pixels = pixels -1      ;so the number of valid pixels is reduced by 1
 good = where(mask)      ;"good" identifies position of valid pixels
 xx= (good mod nbox) - middle  ;x and y coordinate of valid pixels
 yy = fix(good/nbox) - middle    ;relative to the center
 offset = yy*n_x + xx

;Threshold dependent search begins here

 if keyword_set(centroid) then hmin = max(h)
 index = where( h GE hmin, nfound)  ;Valid image pixels are greater than hmin
 if nfound EQ 0 then begin          ;Any maxima found?
  huh = dialog_message(/error,'No maxima exceed input threshold of ' + $
                       string(hmin,'(F9.1)'))
  return
 endif

 for i= 0L, pixels-1 do begin
  stars = where (h[index] GE h[index+offset[i]], nfound)
  if nfound eq 0 then begin  ;Do valid local maxima exist?
   huh = dialog_message(/error,'No maxima exceed input threshold of ' + $
                        string(hmin,'(F9.1)'))
   return
  endif

  index = index[stars]
 endfor

 ix = index mod n_x              ;X index of local maxima
 iy = index/n_x                  ;Y index of local maxima
 ngood = N_elements(index)

 if keyword_set(verbose) then huh=dialog_message(/information,$
      strtrim(ngood,2)+' local maxima located above threshold')

 nstar = 0L        ;NSTAR counts all stars meeting selection criteria
 badround = 0 & badsharp=0  &  badcntrd=0

 if keyword_set(verbose) then print,$
    format='(/8x,a)','     STAR      X      Y     FLUX     SHARP    ROUND'

;Loop over star positions; compute statistics

 for i = 0L,ngood-1 do begin
  temp = float((*image_ptr)[ix[i]-nhalf:ix[i]+nhalf,iy[i]-nhalf:iy[i]+nhalf])
  d = h[ix[i],iy[i]]                  ;"d" is actual pixel intensity

;Compute Sharpness statistic

  sharp1 = (temp[middle,middle] - (total(mask*temp))/pixels)/d

;Compute Roundness statistic

  dx = total( total(temp,2)*c1)
  dy = total( total(temp,1)*c1)
  if (dx LE 0) or (dy LE 0) then begin
   badround = badround + 1
   goto, REJECT           ;Cannot compute roundness
  endif

  around = 2*(dx-dy) / ( dx + dy )    ;Roundness statistic

;Find X centroid

  derivat = shift(temp,-1,0) - temp
  derivat = total( derivat[0:nbox-2,middle-ir:middle+ir],2)
  sumd = total(w*derivat)
  sumxd = total(w*dd*derivat)
  sumxsq = total(w*dd2)

  if ( sumxd GE 0. ) then begin
   badcntrd = badcntrd + 1
   goto,REJECT           ;Cannot compute X centroid
  endif

  dx =sumxsq*sumd/(sumc*sumxd)
  if abs(dx) GT nhalf then begin
   badcntrd = badcntrd + 1
   goto,REJECT           ;X centroid too far from local X maxima
  endif

  xcen = ix[i]-dx               ;Convert back to big (*image_ptr) coordinates

;Find Y centroid

  derivat = shift(temp,0,-1) - temp
  derivat = total( derivat[middle-ir:middle+ir,0:nbox-2], 1 )
  sumd = total( w*derivat )
  sumxd = total( w*dd*derivat )
  sumxsq = total( w*dd2 )
  if (sumxd GE 0) then begin
   badcntrd = badcntrd + 1
   goto, REJECT
  endif

  dy = sumxsq*sumd/(sumc*sumxd)
  if ( abs(dy) GT nhalf ) then begin
   badcntrd = badcntrd + 1
   goto,REJECT
  endif

  ycen = iy[i] - dy

;This star has met all selection criteria.  Print out and save results

  if keyword_set(verbose) then $
     print,FORM = '(12x,i5,2f7.1,f9.1,2f9.2)', $
           nstar, xcen, ycen, d, sharp1, around

  dattemp = fltarr(4,nstar+1,/nozero)
  if nstar gt 0 then dattemp[0,0] = temporary(*result_ptr)
  dattemp[*,nstar] = [xcen,ycen,around,sharp1]
  *result_ptr = temporary(dattemp)

  nstar = nstar+1

REJECT:

 endfor

 if keyword_set(verbose) then begin
  print,' No. of sources rejected by SHARPNESS criteria',badsharp
  print,' No. of sources rejected by ROUNDNESS criteria',badround
  print,' No. of sources rejected by CENTROID  criteria',badcntrd
 endif

;dump all the results back into heap variables for photvis
 *conv_ptr = temporary(h)

;filter out detections that are already user's, then slap user adds on the end
 if uct gt 0 then begin

;filter multi-detections
  chkdat = fix((*result_ptr)[0:1,*]+.5)
  chkusr = fix(user_dat[0:1,*]+.5)
  ind = [-1]
  for i = 0,nstar-1 do begin
   wx = where(chkusr[0,*] eq chkdat[0,i],xct)
   if xct gt 0 then begin
    wy = where(chkusr[1,wx] eq chkdat[1,i],yct)
    if yct eq 0 then begin
     if ind[0] eq -1 then ind = [long(i)] else ind = [ind,long(i)]
    endif
   endif else if ind[0] eq -1 then ind = [long(i)] else ind = [ind,long(i)]
  endfor
  if n_elements(ind) lt nstar then begin
   *result_ptr = temporary((*result_ptr)[*,ind])
   nstar = n_elements(ind)
  endif
   
  dattemp = fltarr(rsz[1],nstar+uct,/nozero)
  aptemp = fltarr(asz[1],nstar+uct)
  if nstar gt 0 then dattemp[0,0] = temporary(*result_ptr)
  dattemp[0,nstar] = temporary(user_dat)
  aptemp[0,nstar] = temporary(user_apers)
  *stat_ptr = fltarr(nstar+uct)
  (*stat_ptr)[nstar:nstar+uct-1] = temporary(user_stat)
  *result_ptr = temporary(dattemp)
  *aper_ptr = temporary(aptemp)
 endif else begin
  if nstar gt 0 then begin
   *stat_ptr = fltarr(nstar)
   *aper_ptr = fltarr(7,nstar)
  endif
 endelse

end
