;+
;   NAME:
;     FLATTEN
;
;   PURPOSE:
;     This procedure reads in a series of images, divides each by a flat field
;     image, and rewrites the resulting processed images to a directory
;     specified by the user.
;
;   CALLING SEQUENCE:
;     flatten
;
;   INPUTS:
;     (NONE) all inputs are GUI-based
;
;   KEYWORDS:
;       INFILES = string array of FITS filenames
;                 (GUI prompt if not specified)
;       OUTDIR = string containing directory to save processed FITS images to
;                (GUI prompt if not specified)
;       FLATDAT = string containing existing FITS file name or structure output
;                 from fds_master.pro.  (GUI prompt if set but invalid)
;       USELOG = boolean (gives GUI prompt) or filename of fds_master log
;                to (re)run
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;
;   OUTPUTS:
;         the original images listed in the designated directory are modified
;         and saved to directory outdir which program will
;         create if it doesn't exist using file_mkdir procedure.
;
;   CALLED PROCEDURES:
;       fits_datmask, merge_mask(), sxaddpar(), logcheck()
;
;   MODIFICATION HISTORY:
;     5-31-02: completed and integrated into pipeline (RAG)
;
;-


pro flatten,outdir=outdir,flatdat=flatdat,infiles=infiles,uselog=uselog,$
             savelog=savelog,no_savelog=no_savelog

 if strlowcase(!version.os_family) eq 'windows' then sls='\' else sls='/'

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1
 endif

;get flatdat if not handed it
 needflat = [1,1]
 if keyword_set(flatdat) then begin
  case size(flatdat,/type) of
   7: begin
    if file_test(flatdat) then needflat[0] = 0 else begin
     print,'ERROR in flatten.pro: File '+flatdat+' does not exist!'
    endelse
   end
   8: begin
    tags = tag_names(flatdat)
    w = where(tags eq 'name' or tags eq 'image' or tags eq 'header' $
              or tags eq 'mask',nt)
    if nt eq 4 then needflat = [0,0] else begin
     print,'ERROR in flatten.pro: flatdat specified is invalid!'
    endelse
   end
   else: print,'ERROR in flatten.pro: flatdat specified is invalid!'
  endcase
 endif

;get flat field filename if needed
 if needflat[0] then begin
  flatdat = dialog_pickfile(/must_exist,$
     title='Select FITS format flat field')
  if flatdat eq '' then begin
   print,'ERROR in flatten.pro: No filename specified!'
   retall
  endif
 endif

;get main image names if not supplied
 if not keyword_set(infiles) then infiles=dialog_pickfile(/multiple,$
    /must_exist,title='Select FITS images to flatten')
 nf = n_elements(infiles)
 if nf le 0 or infiles[0] eq '' then begin
  print,'ERROR in flatten.pro: No files selected!'
  retall
 endif

;get outdir if not provided
 if not keyword_set(outdir) then outdir = dialog_pickfile(/directory,$
    title='Select a directory for output FITS files')
 if outdir eq '' then begin
  print,'ERROR in flatten.pro: No output directory specified!'
  retall
 endif
 if not file_test(strmid(outdir,0,strlen(outdir)-1),/directory) then file_mkdir,outdir

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

;write log if it's called for
 if size(flatdat,/type) eq 7 then begin
  if loghuh then save,filename=savelog,outdir,infiles,flatdat
 endif else begin
  if loghuh then save,filename=savelog,outdir,infiles
 endelse

;****** end user input section

;read in flat field FITS file if needed
 if needflat[1] then begin
  print,'Reading in flat field: '+flatdat
  fits_datmask,flatdat,im,hdr,mask
  flatdat = {name:temporary(flatdat),image:temporary(im),$
             header:temporary(hdr),mask:temporary(mask)}
 endif

;make sure flat image is ok everywhere
 w = where(flatdat.image le 0,nbad)
 if nbad gt 0 then flatdat = {name:temporary(flatdat.name),$
    image:temporary(flatdat.image),header:temporary(flatdat.header),$
    mask:merge_mask(temporary(flatdat.mask),w)}
 nbad = n_elements(flatdat.mask)
 if nbad gt 0 then flatdat.image[flatdat.mask] = 1

 for i = 0,nf-1 do begin

;read in the file with fits_datmask
  print,'Reading '+infiles[i]
  fits_datmask,infiles[i],im,hdr,mask

;get rid of the old path
  fsep = strsplit(infiles[i],sls,/extract)
  fname = fsep[n_elements(fsep)-1]

;merge the masks and divide by the flat
  mask = merge_mask(flatdat.mask,temporary(mask))
  im = float(temporary(im))/flatdat.image

;update header
  systm = systime(/utc)
  sxaddpar,hdr,'HISTORY',systm+' UTC: flattened'
  sxaddpar,hdr,'HISTORY',systm+' UTC: badpix extension updated'

;write the new file
  print,'Writing '+outdir+fname
  fits_datmask,/write,outdir+fname,im,hdr,mask
 endfor

 print,'FLATTEN run completed!'

end
