;+
;    NAME:
;       LINFIND
;
;    PURPOSE:
;       This function calculates the linearity correction for a series of
;       images, returning the result as an anonymous structure.
;
;    CALLING SEQUENCE:
;       result = LINFIND()
;
;    INPUTS:
;       NONE (GUI-based inputs)
;
;    KEYWORDS:
;       INFILES = string array of FITS filenames of linearity sequence
;                 (GUI prompt if not specified)
;       OUTNAME = string containing filename to save LINDAT structure to
;                 (GUI prompt if not specified, can disable by specifying 'NA')
;       USELOG = boolean (gives GUI prompt) or filename of linfind log
;                to (re)run
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;       KSAT = floating constant saturation level (default is dynamic
;              saturation setting pixel by pixel)
;       TOL = difference tolerance for dynamic saturation
;              setting (default tol=200)
;       NO_PLOT = boolean, turn on to kill on-screen plot generation
;       PLOTPS = boolean, or filename...  set to plot to ps file (default file
;                name is 'residuals_linfind.ps')
;
;    OUTPUTS:
;       the output is an anonymous structure of the format:
;        a: zeroth order coefficient image
;        b: first order coefficient image
;        c: second order coefficient image
;        mask: all pixels where the .75 > b/median(b) > 1.5
;              are set to zero
;        sat: an array set to the saturation values for each pixel
;       which also is written to disk using IDL save procedure.
;
;    NOTE:
;       the fits of several different pixels are plotted as a check.
;
;    CALLED PROCEDURES:
;       fits_read, sxpar(), jail(), dysat(), logcheck()
;
;    HISTORY:
;       written 7-18-00: S. T. Megeath
;       last modified: 10-16-00
;       modified 5-29-01: R. Gutermuth
;       1-24-02: did some cleaning, but there's still lots to do! (RAG)
;       2-4-02: finished rehashing this beast, and put in a ton of
;               comments (RAG)
;       2-12-02: switched to standard sxpar.pro usage for getting
;                info from headers (RAG)
;       5-13-02: added logging and log use with logcheck, and switched to
;                saving output to disk using IDL save and anon structure. (RAG)
;       6-26-02: added FLAMINGOS support, as well as generic support for other
;                instruments which have yet to be fully integrated.(RAG)
;       8-19-02: fixed bug with constant sat setting. (RAG)
;       9-3-02: tried again to fix bug with constant sat setting (and
;               succeeded).  Also added plotps keyword. (RAG)
;       10-26-02: added SQIID support. (RAG)
;       2-26-03: fixed sloppy plotting bug if both onscreen plots and plotps
;                are used. (RAG)
;
;-


function linfind,outname=outname,infiles=infiles,ksat=ksat,tol=tol,$
                 no_plot=no_plot,plotps=plotps,$
                 uselog=uselog,no_savelog=no_savelog,savelog=savelog

;****** begin user input section

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1
 endif

;get main image names if not supplied
 if not keyword_set(infiles) then infiles=dialog_pickfile(/multiple,$
    /must_exist,title='Select linearity sequence FITS files')
 nf = n_elements(infiles)
 if nf le 0 or infiles[0] eq '' then begin
  print,'ERROR in linfind.pro: No files selected!'
  retall
 endif

;get outname if not provided
 if not keyword_set(outname) then begin
  huh = 'No'
  while huh ne 'Yes' do begin
   outname = dialog_pickfile(title='Save result as?')
   if file_test(outname) then huh = dialog_message(/question,'File '+outname+$
      'exists...  Overwrite?') else huh = 'Yes'
   if outname eq '' then begin
    print,'Cancel: Not writing result to disk...'
    huh = 'Yes'
   endif
  endwhile
 endif

 if keyword_set(plotps) then begin
  if size(plotps,/type) ne 7 then plotps = 'residuals_linfind.ps'
 endif else plotps = 0

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

 if NOT keyword_set(ksat) then begin
  sat=1.e6 & ksat = 0
 endif
 if not keyword_set(tol) then tol=200

;write log if it's called for
 if loghuh then save,filename=savelog,outname,infiles,ksat,tol,no_plot,plotps

;****** end user input section

;load in data, (normalized by coadds for STELIRCam), and create
;   array of itimes.

 begintime = systime(1)

 for i=0,nf-1 do begin
  print,'Reading '+infiles[i]
  fits_read,infiles[i],data,hdr

  if i eq 0 then begin
   sz = size(data)

;Instrument is?
   ins = strtrim(sxpar(hdr,'INSTRUME'),2)
   tkey = 'ITIME' & sflg = 0

;FLAMINGOS jun01, INT_S...  feb02, EXP_TIME...  MAKE UP YOUR MIND!!!!

   case ins of
    'SQIID': tkey = 'INT_SH'                 ;sqiid
    'FLAMINGOS': tkey = 'EXP_TIME'           ;flamingos
    'ADS SBRC Camera': sflg = 1              ;stelircam
    else: print,'WARNING: Data from unrecognized/unsupported instrument!  '+$
       'Better cross your fingers!'          ;others _should_ work, but... ;)
   endcase

;   datacube = make_array(sz[1],sz[2],nf,type=sz[sz[0]+1],/nozero)
   datacube = make_array(nf,sz[1],sz[2],type=sz[sz[0]+1],/nozero)
   time = fltarr(nf)
   a = fltarr(sz[1],sz[2])
   b = a & c = a & satmap = a
  endif

  itime = float(sxpar(hdr,tkey))

;stelircam specific stuff
  if sflg then begin
   coadds = fix(sxpar(hdr,'CO_ADDS'))
   ndata = jail(temporary(data))
;   datacube[*,*,i] = temporary(ndata)/float(coadds[0])
   datacube[i,*,*] = temporary(ndata)/float(coadds[0])
  endif else begin
;end stelircam specific stuff
;   datacube[*,*,i] = temporary(data)
   datacube[i,*,*] = temporary(data)
  endelse

  time[i] = itime

 endfor

 x = time
 srt = sort(x)
 newx = x[srt]

 old_pmulti= !p.multi
 !p.multi = [0,4,4]

 if keyword_set(plotps) then begin
  old_device = !d.name
  set_plot,'PS'
  device,filename=plotps
 endif

 for j=0,sz[2]-1 do begin
  for i=0,sz[1]-1 do begin

;   y = datacube[i,j,*]
   y = datacube[*,i,j]
   newy = y[srt]

;mod-RAG- using a dynamic saturation setting based on slope between
;         successive data points  (through dysat.pro)
   if not keyword_set(ksat) then begin
    chop = dysat(newx,newy,tol)
    if chop eq max(x) then sat = 1.e6 else $
       sat = mean(newy[where(x eq chop)]) > 0
   endif else begin
    sat = ksat

;mod-RAG- cobbed soln to make constant sat value work
    gw = where(y lt sat,ng)
    if ng gt 0 then chop = x[(where(y eq max(y[gw])))[0]] else chop = min(x)
   endelse

;chop saturated data
   if chop le min(x) then begin

;if a pixel really misbehaves, this should've caught it, but negative values
; still loused things up, so I made a more robust solution.  (RAG)
;    y = fltarr(nf)*0.
;    index = where(y le sat)

    a[i,j] = 0.
    b[i,j] = 0.
    c[i,j] = 0.
    satmap[i,j] = 0.
   endif else begin

;do second order polynomial fit
    index = where(x le chop,ns)
    result = poly_fit(x[index],y[index],2)
    a[i,j] = result[0]
    b[i,j] = result[1]
    c[i,j] = result[2]
    satmap[i,j] = sat
   endelse

   if i eq 0 then print,'Beginning row '+strtrim(j,2)

;show representative plots

   if (total(i eq (indgen(4)+1)*fix(sz[1]/5.)) + $
      total(j eq (indgen(4)+1)*fix(sz[2]/5.)) ge 2) then begin
    yprime = a[i,j]+b[i,j]*newx+c[i,j]*newx^2

    if keyword_set(plotps) then begin
     if not keyword_set(no_plot) then current_pmulti = !p.multi
     plot,newx,(newy-yprime)/newy,psym=2,yrange=[-0.20,0.20]
     plot,newx,newy,psym=2
     oplot,newx,yprime
     set_plot,old_device
    endif

    if not keyword_set(no_plot) then begin
     if keyword_set(plotps) then !p.multi = current_pmulti
     plot,newx,(newy-yprime)/newy,psym=2,yrange=[-0.20,0.20]
     plot,newx,newy,psym=2
     oplot,newx,yprime
    endif

    if keyword_set(plotps) then set_plot,'PS'
   endif

  endfor
 endfor

 if keyword_set(plotps) then begin
  device,/close_file
  set_plot,old_device
 endif

;make mask image
 mask = a*0+1

;second time we've changed/loosened the automasker "algorithm"...
;feel free to replace the mask that comes out of this if you don't like.
; index = where(abs(b-median(b)) gt 5.*stdev(b))
; index = where(b/median(b) gt 1.5 or b/median(b) lt .75,nbad)
 index = where(b/median(b) gt 1.6 or b/median(b) lt .4,nbad)

 if nbad gt 0 then mask[index] = 0

;return output structure
 lindat = {a:a,b:b,c:c,mask:mask,sat:satmap}

;write results with IDL save procedure
 if outname ne '' and outname ne 'NA' then begin
  print,'Saving lindat structure...'
  save,filename=outname,lindat
 endif else print,'Not saving LINDAT structure.'

;cleanup and return results
 !p.multi = old_pmulti
 print,'LINFIND run completed!'
 return,lindat

end


