;+
;    NAME:
;       MERGE_MASK
;
;    PURPOSE:
;       This function merges two masks into one, and returns the resulting 
;       vector of bad pixels, or a -1 if there were no valid bad pixels.
;
;    CALLING SEQUENCE:
;       result = merge_mask(mask1,mask2)
;
;    INPUTS:
;       mask1 = vector of bad pixels or 2D binary table
;       mask2 = other mask vector or table to be merged with mask1
;
;    KEYWORDS:
;       AGREE = set to have merged mask only be those pixels present in BOTH
;               masks (default is to include pixels present in EITHER mask)
;
;    OUTPUTS:
;       result is the merged vector of bad pixels, or -1
;
;    OPTIONAL OUTPUTS:
;       nmask: longword integer of number of bad pixels in merged mask
;
;    CALLED PROCEDURES:
;       No non-standard programs are called
;
;    HISTORY:
;       5-31-02: completed and integrated into pipeline (RAG)
;       7-3-02: Added support for greater than 32767 mask elements, 
;               as demanded by FLAMINGOS data...  This of course meant 
;               a total reworking of this program...  :(  Also sorted 
;               output mask, just for fun. (RAG)
;
;-


function merge_mask,mask1,mask2,nmask,agree=agree

;get info on the input masks
 sm1 = size(mask1) & sm2 = size(mask2)
 n1 = sm1[sm1[0]+2] & n2 = sm2[sm2[0]+2]

;fix them if they're not as expected, and get out if they're screwy...
 if sm1[0] gt 1 then begin
  w = where(mask1 eq 0 or mask1 eq 1,nw)
  if nw eq n1 then mask1 = where(temporary(mask1) eq 0,n1) else begin
   print,'ERROR in merge_mask.pro: Invalid data in mask1!'
   nmask = 0l & return,-1
  endelse
 endif
 if sm2[0] gt 1 then begin
  w = where(mask2 eq 0 or mask2 eq 1,nw)
  if nw eq n1 then mask2 = where(temporary(mask2) eq 0,n1) else begin
   print,'ERROR in merge_mask.pro: Invalid data in mask2!'
   nmask = 0l & return,-1
  endelse
 endif

;if we don't have any masked elements at this point, get out...
 if n1+n2 eq 0 then begin
  nmask = 0l
  return,-1
 endif

;ditch redundant values...  they just cause problems...
 mask1 = mask1[uniq(mask1,sort(mask1))]
 mask2 = mask2[uniq(mask2,sort(mask2))]
 sm1 = size(mask1) & sm2 = size(mask2)
 n1 = sm1[sm1[0]+2] & n2 = sm2[sm2[0]+2]

;any invalid masks?  depending on mode, we deal with them differently

;if we have single element mask(s), could be a -1...  need to check
;that and get out if we have one.
 if mask1[0] eq -1 or mask2[0] eq -1 then begin
  if keyword_set(agree) then begin
   nmask = 0l & return,-1
  endif else begin
   if mask1[0] eq -1 then result = mask2
   if mask2[0] eq -1 then result = mask1
   nmask = n_elements(result)
   return,result
  endelse
 endif

 if keyword_set(agree) then begin

;single element masks muff up "agree" array combination 
;method so we merge single element masks differently here.
  case 1 of
   n1 eq 1: begin
    w = where(mask2 eq mask1[0],nmask)
    if nmask eq 0 then return,-1 else return,mask1
   end
   n2 eq 1: begin
    w = where(mask1 eq mask2[0],nmask)
    if nmask eq 0 then return,-1 else return,mask2
   end
   else: break
  endcase

;finally! merge the masks using the "agree" method
  temp = [mask1,mask2]
  temp = temp[sort(temp)]
  diff = temp - shift(temp,-1)
  w = where(diff eq 0,nmask)
  if nmask gt 0 then result = temp[w] else result = -1 
 endif else begin

;if both are single element, default array combination
;method gets muffed...  deal with it here...
  if n1+n2 eq 2 then begin
   if mask1[0] eq mask2[0] then begin
    nmask = 1l & return,mask1
   endif else begin
    result = [mask1,mask2]
    nmask = 2l & return,result[sort(result)]
   endelse
  endif

;finally! merge the masks using the default method
  temp = [mask1,mask2]
  result = temp[uniq(temp,sort(temp))]
  nmask = n_elements(result)
 endelse
  
 return,result

end
