;+
;   name:
;   rob_reg_gui.pro
;
;   purpose:
;       This function(and associated GUI) allows the selection
;       of a sub-region of master, and if necessary, move the
;       corresponding sub-region selected in slave to facilitate
;       mosaic creation.  The selections are then tracked using
;       xyoff.pro.
;
;
;   calling sequence:
;       result = rob_reg_gui(master,slave)
;
;   inputs:
;       master = reference image
;       slave = image to track with respect to master
;
;   keywords:
;       redo = set to create REDO PREVIOUS button (used with assemble.pro)
;       scl = 2x2 float array for min and max of both images (used for 
;             persistent scaling
;
;   output:
;       2-element vector containing x and y offsets between master
;       and slave to feed to mosaic_two.pro
;
;   To use rob_reg_gui.pro:
;       1) use the slider bars to scale the images displayed;
;       2) click and drag on the master image to select the region to track on;
;       3) if necessary, click and drag the corresponding box on the slave
;              image to select the region that matches the one selected
;              in the master;
;       4) hit the SUBMIT button to enter your selection and allow
;              the computer to determine the offset;
;       5) if using through assemble.pro, hit the REDO PREVIOUS button to
;              jump back to the previous master and slave to re-register them
;              (can only go back one image!!)
;
;   called programs:
;
;   history:
;       initial coding 6-7-01: R. Gutermuth
;       completed     6-12-01: R. Gutermuth
;       6-18-01: fixed box redrawing bug that occured on image rescale (RAG)
;       6-20-01: added pixel value label (RAG)
;       7-2-01 : added "persistent scaling" feature to keep image scaling
;                between calls (RAG)
;       7-3-01 : added rescaling of large images (RAG)
;       6-12-02: ditched writing temp file, transfer data with pointers instead.
;                (RAG)
;       7-9-02: added permanent 'Kill it' button so that if things are not 
;               desirable, one can just get the window killed and a retall (RAG)
;       2-4-03: fixed stupid bug with worthless help button...  must've copied
;               the code directly from photvis or something...  (RAG)
;
;-


;*****************************************************
;Event Handler
;*****************************************************

pro rob_reg_gui_event,ev

widget_control,ev.top,get_uvalue=ptr_rruv

rruv = temporary(*ptr_rruv)

case ev.id of

 rruv.mmin: begin
  widget_control,ev.id,get_value=in
  widget_control,rruv.mmax,get_value=mx
  if in ge mx then rruv.mscr = byte(rruv.master*0) else begin
   rruv.mscr = bytscl(rruv.master,min=in,max=mx)
  endelse
  wset,rruv.mwn
  tv,rruv.mscr
  if abs(rruv.ds[0,1]-rruv.ds[0,2]) gt 5 and $
     abs(rruv.ds[1,1]-rruv.ds[1,2]) gt 5 then begin
   plots,[rruv.ds[0,1],rruv.ds[0,1],rruv.ds[0,2],rruv.ds[0,2],rruv.ds[0,1]],$
      [rruv.ds[1,1],rruv.ds[1,2],rruv.ds[1,2],rruv.ds[1,1],rruv.ds[1,1]],$
      /device
  endif
 end

 rruv.mmax: begin
  widget_control,ev.id,get_value=in
  widget_control,rruv.mmin,get_value=mn
  if in le mn then rruv.mscr = byte(rruv.master*0) else begin
   rruv.mscr = bytscl(rruv.master,min=mn,max=in)
  endelse
  wset,rruv.mwn
  tv,rruv.mscr
  if abs(rruv.ds[0,1]-rruv.ds[0,2]) gt 5 and $
     abs(rruv.ds[1,1]-rruv.ds[1,2]) gt 5 then begin
   plots,[rruv.ds[0,1],rruv.ds[0,1],rruv.ds[0,2],rruv.ds[0,2],rruv.ds[0,1]],$
      [rruv.ds[1,1],rruv.ds[1,2],rruv.ds[1,2],rruv.ds[1,1],rruv.ds[1,1]],$
      /device
  endif
 end

 rruv.smin: begin
  widget_control,ev.id,get_value=in
  widget_control,rruv.smax,get_value=mx
  if in ge mx then rruv.sscr = byte(rruv.slave*0) else begin
   rruv.sscr = bytscl(rruv.slave,min=in,max=mx)
  endelse
  wset,rruv.swn
  tv,rruv.sscr
  xsh = rruv.ds[0,5]
  ysh = rruv.ds[1,5]
  if abs(rruv.ds[0,1]-rruv.ds[0,2]) gt 5 and $
     abs(rruv.ds[1,1]-rruv.ds[1,2]) gt 5 then begin
   plots,[rruv.ds[0,1]+xsh,rruv.ds[0,1]+xsh,rruv.ds[0,2]+xsh,rruv.ds[0,2]$
      +xsh,rruv.ds[0,1]+xsh],[rruv.ds[1,1]+ysh,rruv.ds[1,2]+ysh,rruv.ds[1,2]$
      +ysh,rruv.ds[1,1]+ysh,rruv.ds[1,1]+ysh],/device
  endif
 end

 rruv.smax: begin
  widget_control,ev.id,get_value=in
  widget_control,rruv.smin,get_value=mn
  if in le mn then rruv.sscr = byte(rruv.slave*0) else begin
   rruv.sscr = bytscl(rruv.slave,min=mn,max=in)
  endelse
  wset,rruv.swn
  tv,rruv.sscr
  xsh = rruv.ds[0,5]
  ysh = rruv.ds[1,5]
  if abs(rruv.ds[0,1]-rruv.ds[0,2]) gt 5 and $
     abs(rruv.ds[1,1]-rruv.ds[1,2]) gt 5 then begin
   plots,[rruv.ds[0,1]+xsh,rruv.ds[0,1]+xsh,rruv.ds[0,2]+xsh,rruv.ds[0,2]$
      +xsh,rruv.ds[0,1]+xsh],[rruv.ds[1,1]+ysh,rruv.ds[1,2]+ysh,rruv.ds[1,2]$
      +ysh,rruv.ds[1,1]+ysh,rruv.ds[1,1]+ysh],/device
  endif
 end

 rruv.mw: begin

  case 1 of
   ev.x gt rruv.msz[1]-1: nx = rruv.msz[1]-1
   ev.x lt 0: nx = 0
   else: nx = ev.x
  endcase

  case 1 of
   ev.y gt rruv.msz[2]-1: ny = rruv.msz[2]-1
   ev.y lt 0: ny = 0
   else: ny = ev.y
  endcase

  if nx gt rruv.ssz[1]-1 then xd=rruv.ssz[1]-1-nx else xd=0
  if ny gt rruv.ssz[2]-1 then yd=rruv.ssz[2]-1-ny else yd=0

  case ev.type of

   0: begin
    if rruv.ds[0,0] eq 0 then begin
     wset,rruv.mwn
     tv,rruv.mscr
    endif
    rruv.ds[0,*] = [0,nx,0,0,0,xd]
    rruv.ds[1,*] = [0,ny,0,0,0,yd]
   end

   1: begin
    rruv.ds[0,*] = [-1,rruv.ds[0,1],nx,0,0,rruv.ds[0,5]]
    rruv.ds[1,*] = [-1,rruv.ds[1,1],ny,0,0,rruv.ds[1,5]]
    if abs(rruv.ds[0,2]-rruv.ds[0,1]) gt 5 or $
       abs(rruv.ds[1,2]-rruv.ds[1,1]) gt 5 then begin
     widget_control,rruv.ok,sensitive=1
     widget_control,rruv.sw,sensitive=1
    endif else begin
     wset,rruv.mwn
     tv,rruv.mscr
     wset,rruv.swn
     tv,rruv.sscr
     widget_control,rruv.ok,sensitive=0
     widget_control,rruv.sw,sensitive=0
    endelse
   end

   2: begin
    widget_control,rruv.xl,set_value='X: '+strtrim(nx,2)
    widget_control,rruv.yl,set_value='Y: '+strtrim(ny,2)
    widget_control,rruv.il,set_value='Value: '+strtrim(rruv.master[nx,ny],2)
    if rruv.ds[0,0] eq 0 then begin
     if abs(rruv.ds[0,1]-nx)+1 le rruv.ssz[1] and $
        abs(rruv.ds[1,1]-ny)+1 le rruv.ssz[2] then begin
      wset,rruv.mwn
      tv,rruv.mscr
      plots,[rruv.ds[0,1],rruv.ds[0,1],nx,nx,rruv.ds[0,1]],$
         [rruv.ds[1,1],ny,ny,rruv.ds[1,1],rruv.ds[1,1]],/device
      if nx gt rruv.ds[0,1] then rruv.ds[0,5] = xd
      if ny gt rruv.ds[1,1] then rruv.ds[1,5] = yd
      wset,rruv.swn
      tv,rruv.sscr
      plots,[rruv.ds[0,1]+rruv.ds[0,5],rruv.ds[0,1]+rruv.ds[0,5],$
         nx+rruv.ds[0,5],nx+rruv.ds[0,5],rruv.ds[0,1]+rruv.ds[0,5]],$
         [rruv.ds[1,1]+rruv.ds[1,5],ny+rruv.ds[1,5],ny+rruv.ds[1,5],$
         rruv.ds[1,1]+rruv.ds[1,5],rruv.ds[1,1]+rruv.ds[1,5]],/device
     endif
    endif
   end

  endcase
 end

 rruv.sw: begin

  case 1 of
   ev.x gt rruv.ssz[1]-1: nx = rruv.ssz[1]-1
   ev.x lt 0: nx = 0
   else: nx = ev.x
  endcase

  case 1 of
   ev.y gt rruv.ssz[2]-1: ny = rruv.ssz[2]-1
   ev.y lt 0: ny = 0
   else: ny = ev.y
  endcase

  x1 = min(rruv.ds[0,1:2]) & x2 = max(rruv.ds[0,1:2])
  y1 = min(rruv.ds[1,1:2]) & y2 = max(rruv.ds[1,1:2])

  if rruv.ds[0,0] eq 0 then begin
   xsh = nx-rruv.ds[0,3]+rruv.ds[0,5] & ysh = ny-rruv.ds[1,3]+rruv.ds[1,5]

   case 1 of
    x2+xsh gt rruv.ssz[1]-1: nx=fix(rruv.ssz[1]-1+rruv.ds[0,3]-rruv.ds[0,5]-x2)
    x1+xsh lt 0: nx=fix(rruv.ds[0,3]-rruv.ds[0,5]-x1)
    else: nx = nx
   endcase

   case 1 of
    y2+ysh gt rruv.ssz[2]-1: ny=fix(rruv.ssz[2]-1+rruv.ds[1,3]-rruv.ds[1,5]-y2)
    y1+ysh lt 0: ny=fix(rruv.ds[1,3]-rruv.ds[1,5]-y1)
    else: ny = ny
   endcase

   xsh = nx-rruv.ds[0,3]+rruv.ds[0,5] & ysh = ny-rruv.ds[1,3]+rruv.ds[1,5]
  endif

  case ev.type of

   0: begin
    if rruv.ds[0,0] eq 0 then begin
     wset,rruv.swn
     tv,rruv.sscr
    endif
    if (nx le x2+rruv.ds[0,5] and nx ge x1+rruv.ds[0,5]) and $
       (ny le y2+rruv.ds[1,5] and ny ge y1+rruv.ds[1,5]) then begin
     rruv.ds[*,0] = [0,0]
     rruv.ds[*,3] = [nx,ny]
    endif
   end

   1: begin
    rruv.ds[*,0] = [-1,-1]
    rruv.ds[*,5] = rruv.ds[*,5]+[nx,ny]-rruv.ds[*,3]
   end

   2: begin
    widget_control,rruv.xl,set_value='X: '+strtrim(nx,2)
    widget_control,rruv.yl,set_value='Y: '+strtrim(ny,2)
    widget_control,rruv.il,set_value='Value: '+strtrim(rruv.slave[nx,ny],2)
    if rruv.ds[0,0] eq 0 then begin
     wset,rruv.swn
     tv,rruv.sscr
     plots,[x1+xsh,x1+xsh,x2+xsh,x2+xsh,x1+xsh],$
           [y1+ysh,y2+ysh,y2+ysh,y1+ysh,y1+ysh],/device
    endif
   end

  endcase

 end

 rruv.help: huh = dialog_message(/info,dialog_parent=ev.top, $
    "Email questions, bug reports, etc. to rguter@astro.pas.rochester.edu")

 rruv.ok: begin
  widget_control,rruv.mmin,get_value=mmin
  widget_control,rruv.mmax,get_value=mmax
  widget_control,rruv.smin,get_value=smin
  widget_control,rruv.smax,get_value=smax

  x1 = min(rruv.ds[0,1:2]) & x2 = max(rruv.ds[0,1:2])
  y1 = min(rruv.ds[1,1:2]) & y2 = max(rruv.ds[1,1:2])
  ss = [x1,y1,x2,y2,rruv.ds[0,5],rruv.ds[1,5],1,rruv.zflg,mmin,mmax,smin,smax]
  ss[0:5] = ss[0:5]*rruv.zflg

  rruv.ss = reform(ss,2,6)
  
;  openw,one,'~/anger.tmp',/get
;  writeu,one,ss
;  free_lun,one

  widget_control,ev.top,/destroy
;  return
 end

 rruv.redo: begin
  widget_control,rruv.mmin,get_value=mmin
  widget_control,rruv.mmax,get_value=mmax
  widget_control,rruv.smin,get_value=smin
  widget_control,rruv.smax,get_value=smax
  ss = fltarr(12)
  ss[8:11] = [mmin,mmax,smin,smax]
;  openw,one,'~/anger.tmp',/get
;  writeu,one,ss
;  free_lun,one

  rruv.ss = reform(ss,2,6)
  widget_control,ev.top,/destroy
;  return
 end

 rruv.exit: begin
  widget_control,ev.top,/destroy
  retall
 end

endcase

*ptr_rruv = temporary(rruv)
;widget_control,ev.top,set_uvalue=rruv,/no_copy

end

;**********************************************************
;GUI Initialization Procedure
;**********************************************************

function rob_reg_gui,master,slave,redo=redo,scl=scl;,test=test

mmo = moment(master,sdev=msd)
smo = moment(slave,sdev=ssd)
mmo = [mmo[0],msd]
smo = [smo[0],ssd]

if not keyword_set(scl) then begin
 scl = fltarr(2,2)
 scl[0,*] = [mmo[0]-2*mmo[1],mmo[0]+2*mmo[1]]
 scl[1,*] = [smo[0]-2*smo[1],smo[0]+2*smo[1]]
endif else begin
 if (scl[0,0] lt mmo[0]-10*mmo[1] or scl[0,0] gt mmo[0]+10*mmo[1]) then $
    scl[0,0] = mmo[0]-2*mmo[1]
 if (scl[0,1] lt mmo[0]-10*mmo[1] or scl[0,1] gt mmo[0]+10*mmo[1]) then $
    scl[0,1] = mmo[0]+2*mmo[1]
 if (scl[1,0] lt smo[0]-10*smo[1] or scl[1,0] gt smo[0]+10*smo[1]) then $
    scl[1,0] = smo[0]-2*smo[1]
 if (scl[1,1] lt smo[0]-10*smo[1] or scl[1,1] gt smo[0]+10*smo[1]) then $
    scl[1,1] = smo[0]+2*smo[1]
endelse

main = widget_base(/column,title='rob_reg_gui')
dmain = widget_base(main,/row)

mbase = widget_base(dmain,/column)
sbase = widget_base(dmain,/column)
bbase = widget_base(main,/row)

mmin = cw_fslider(mbase,/edit,min=mmo[0]-10*mmo[1],max=mmo[0]+10*mmo[1],$
      value=scl[0,0],title='Master Min',/frame)
mmax = cw_fslider(mbase,/edit,min=mmo[0]-10*mmo[1],max=mmo[0]+10*mmo[1],$
      value=scl[0,1],title='Master Max',/frame)
smin = cw_fslider(sbase,/edit,min=smo[0]-10*smo[1],max=smo[0]+10*smo[1],$
      value=scl[1,0],title='Slave Min',/frame)
smax = cw_fslider(sbase,/edit,min=smo[0]-10*smo[1],max=smo[0]+10*smo[1],$
      value=scl[1,1],title='Slave Max',/frame)

msz = size(master)
ssz = size(slave)
zflg = 1

if msz[1] gt 512 and msz[2] gt 512 and ssz[1] gt 512 and ssz[2] gt 512 $
   then begin
 mxfac = (msz[1]-(msz[1] mod 512))/512
 zflg = mxfac
 dmaster = congrid(master,fix(msz[1]/mxfac),fix(msz[2]/mxfac))
 dslave = congrid(slave,fix(ssz[1]/mxfac),fix(ssz[2]/mxfac))
 msz = size(dmaster)
 ssz = size(dslave)
endif else begin
 dmaster = master
 dslave = slave
endelse

mscr = bytscl(dmaster,min=scl[0,0],max=scl[0,1])
sscr = bytscl(dslave,min=scl[1,0],max=scl[1,1])

if msz[1] gt 512 then sx = 512 else sx = msz[1]
if msz[2] gt 512 then sy = 512 else sy = msz[2]
mwin = widget_draw(mbase,/scroll,retain=2,/motion_events,/button_events,$
      xsize=msz[1],ysize=msz[2],x_scroll_size=sx,y_scroll_size=sy)
;if zflg gt 1 then begin
; mzwin = widget_draw(mbase,/retain,xsize=200,ysize=200)
;endif else mzwin=0l

if ssz[1] gt 512 then sx = 512 else sx = ssz[1]
if ssz[2] gt 512 then sy = 512 else sy = ssz[2]
swin = widget_draw(sbase,/scroll,retain=2,/motion_events,/button_events,$
      xsize=ssz[1],ysize=ssz[2],x_scroll_size=sx,y_scroll_size=sy)
;if zflg gt 1 then begin
; szwin = widget_draw(sbase,/retain,xsize=200,ysize=200)
;endif else szwin=0l

help = widget_button(bbase,value='HELP')
ok = widget_button(bbase,value='SUBMIT',sensitive=0)

if keyword_set(redo) then begin
 rd = widget_button(bbase,value='REDO PREVIOUS')
endif else rd = 0l
;if keyword_set(test) then begin
exit = widget_button(bbase,value='KILL IT')
;endif else exit = 0l

xl = widget_label(bbase,value='X: ',/dynamic_resize)
yl = widget_label(bbase,value='Y: ',/dynamic_resize)
il = widget_label(bbase,value='Value: ',/dynamic_resize)

ss = fltarr(2,6)
ds = fltarr(2,6)
ds[*,0] = -1

rruv = {m:main,mb:mbase,sb:sbase,mmin:mmin,mmax:mmax,smin:smin,smax:smax,$
        mw:mwin,sw:swin,help:help,ok:ok,xl:xl,yl:yl,ss:ss,master:dmaster,$
        slave:dslave,mscr:mscr,sscr:sscr,ds:ds,mmo:mmo,smo:smo,msz:msz,$
        ssz:ssz,mwn:0l,swn:0l,exit:exit,redo:rd,il:il,zflg:zflg}

widget_control,main,/realize

widget_control,mwin,get_value=mw
rruv.mwn=mw
wset,mw
tv,mscr

widget_control,swin,get_value=sw
rruv.swn=sw
wset,sw
tv,sscr
widget_control,swin,sensitive=0

ptr_rruv = ptr_new(temporary(rruv))

widget_control,main,set_uvalue=ptr_rruv
xmanager,'rob_reg_gui',main

rruv = temporary(*ptr_rruv)
ptr_free,ptr_rruv

return,rruv.ss

end



