;+
;   NAME:
;     SKY_SUB
;
;   PURPOSE:
;     This procedure reads in a series of images, subtracts a sky frame from
;     each image, and rewrites the resulting processed images to a directory
;     specified by the user.
;
;   CALLING SEQUENCE:
;     sky_sub
;
;   INPUTS:
;     (NONE) all inputs are GUI-based
;
;   KEYWORDS:
;       INFILES = string array of FITS filenames
;                 (GUI prompt if not specified)
;       OUTDIR = string containing directory to save processed FITS images to
;                (GUI prompt if not specified)
;       SKYDAT = string containing existing FITS file name or structure output
;                 from fds_master.pro.  (GUI prompt if not set or invalid)
;       USELOG = boolean (gives GUI prompt) or filename of sky_sub log
;                to (re)run
;       FDSLOG = boolean (gives GUI prompt) or filename of fds_master log
;                to use for inputs (needs /sky to be set, will still need 
;                outdir and savelog or no_savelog to be specified!)
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;
;   OUTPUTS:
;         the original images chosen are modified
;         and saved to directory outdir which program will
;         create if it doesn't exist using file_mkdir procedure.
;
;   CALLED PROCEDURES:
;       fits_datmask, merge_mask(), sxaddpar(), logcheck()
;
;   MODIFICATION HISTORY:
;     5-31-02: completed and integrated into pipeline (RAG)
;     2-26-03: added fdslog keyword for using fds_master sky frame logs to 
;              get inputs. (RAG)
;
;-


pro sky_sub,outdir=outdir,skydat=skydat,infiles=infiles,uselog=uselog,$
             savelog=savelog,no_savelog=no_savelog,fdslog=fdslog

 if strlowcase(!version.os_family) eq 'windows' then sls='\' else sls='/'

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1 & delvarx,fdslog
 endif

;fdslog takes fds_master sky frame logs to set some of the inputs
 check1 = [keyword_set(no_savelog),keyword_set(savelog),keyword_set(infiles),keyword_set(outdir)]
 huh2 = logcheck(fdslog)
 if huh2 then begin
  if check1[0] then old_no_savelog = no_savelog else old_no_savelog = 0
  if check1[1] then old_savelog = savelog else old_savelog = ''
  if check1[2] then old_infiles = infiles else old_infiles = ''
  if check1[3] then old_outdir = outdir else old_outdir = ''
  restore,fdslog
  flag = n_elements(sky)
  if flag then begin
   if sky then begin
    skydat = outname
   endif else flag = 0
  endif
  if not flag then begin
   print,'ERROR in sky_sub: log not for an fds_master sky frame! Ignoring...'
   if check1[2] then infiles = old_infiles
   if check1[3] then outdir = old_outdir
  endif
  if check1[0] then no_savelog = old_no_savelog
  if check1[1] then savelog = old_savelog
 endif

;get skydat if not handed it
 needsky = [1,1]
 if keyword_set(skydat) then begin
  case size(skydat,/type) of
   7: begin
    if file_test(skydat) then needsky[0] = 0 else begin
     print,'ERROR in sky_sub.pro: File '+skydat+' does not exist!'
    endelse
   end
   8: begin
    tags = tag_names(skydat)
    w = where(tags eq 'name' or tags eq 'image' or tags eq 'header' $
              or tags eq 'mask',nt)
    if nt eq 4 then needsky = [0,0] else begin
     print,'ERROR in sky_sub.pro: skydat specified is invalid!'
    endelse
   end
   else: print,'ERROR in sky_sub.pro: skydat specified is invalid!'
  endcase
 endif

;get sky frame filename if needed
 if needsky[0] then begin
  skydat = dialog_pickfile(/must_exist,$
     title='Select FITS format sky frame')
  if skydat eq '' then begin
   print,'ERROR in sky_sub.pro: No filename specified!'
   retall
  endif
 endif

;get main image names if not supplied
 if not keyword_set(infiles) then infiles=dialog_pickfile(/multiple,$
    /must_exist,title='Select FITS images to sky subtract')
 nf = n_elements(infiles)
 if nf le 0 or infiles[0] eq '' then begin
  print,'ERROR in sky_sub.pro: No files selected!'
  retall
 endif

;get outdir if not provided
 if not keyword_set(outdir) then outdir = dialog_pickfile(/directory,$
    title='Select a directory for output FITS files')
 if outdir eq '' then begin
  print,'ERROR in sky_sub.pro: No output directory specified!'
  retall
 endif
 if not file_test(strmid(outdir,0,strlen(outdir)-1),/directory) then file_mkdir,outdir

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

;write log if it's called for
 if size(skydat,/type) eq 7 then begin
  if loghuh then save,filename=savelog,outdir,infiles,skydat
 endif else begin
  if loghuh then save,filename=savelog,outdir,infiles
 endelse

;****** end user input section

;read in sky frame FITS file if needed
 if needsky[1] then begin
  print,'Reading in sky frame: '+skydat
  fits_datmask,skydat,im,hdr,mask
  skydat = {name:temporary(skydat),image:temporary(im),$
             header:temporary(hdr),mask:temporary(mask)}
 endif

;make sure sky frame is ok everywhere
 w = where(skydat.image lt 0,nbad)
 if nbad gt 0 then skydat = {name:temporary(skydat.name),$
    image:temporary(skydat.image),header:temporary(skydat.header),$
    mask:merge_mask(temporary(skydat.mask),w)}
 nbad = n_elements(skydat.mask)
 if nbad gt 0 then skydat.image[skydat.mask] = 0
 skymed = median(skydat.image)

 for i = 0,nf-1 do begin

;read in the file with fits_datmask
  print,'Reading '+infiles[i]
  fits_datmask,infiles[i],im,hdr,mask

;get rid of the old path
  fsep = strsplit(infiles[i],sls,/extract)
  fname = fsep[n_elements(fsep)-1]

;merge the masks and divide by the flat
  mask = merge_mask(skydat.mask,temporary(mask))
  im = float(temporary(im)) - skydat.image + skymed

;update header
  systm = systime(/utc)
  sxaddpar,hdr,'HISTORY',systm+' UTC: sky subtracted'
  sxaddpar,hdr,'HISTORY',systm+' UTC: badpix extension updated'

;write the new file
  print,'Writing '+outdir+fname
  fits_datmask,/write,outdir+fname,im,hdr,mask
 endfor

 print,'SKY_SUB run completed!'

end
