;+
; NAME:
;    stelir_sort.pro
;
; PURPOSE:
;    Sorts a directory of STELIRCam data
;
; CATEGORY:
;    Data processing
;
; CALLING SEQUENCE:
;    stelir_sort,dirname=dirname,/dark,/lin,/flat,/data
;
; INPUTS:
;    (NONE)
;
; KEYWORD PARAMETERS:
;    dirname = (string) directory with STELIRCam FITS images
;    dark = switch on if data are dark frames
;    lin = switch on if data are linearity images
;    flat = switch on if data are flat field images
;    data = switch on if data are actual data images!!
;
; OUTPUTS:
;    Writes images to appropriate directories
;
; COMMON BLOCKS:
;    Ha!  Surely you jest!  ;)
;
; PROCEDURE CALLS:
;    fits_datmask,sxpar()
;
; SIDE EFFECTS:
;    Creates directories for receiving sorted images
;
; MODIFICATION HISTORY:
;    2-1-02: Pretty much finished (RAG)
;    7-31-02: added fits_datmask usage, a little late, though...  (RAG)
;
;-


pro stelir_sort,dirname=dirname,dark=dark,lin=lin,flat=flat,data=data

;get dirname if not supplied
 if not keyword_set(dirname) then dirname = dialog_pickfile(/directory,title=$
   'Select a STELIRCam directory to sort')
 if dirname eq '' then begin
  print,'ERROR in stelir_sort.pro: No directory selected!'
  retall
 endif

;get image filenames
 f = findfile(dirname+'*?.?*',count=nf)
 if nf eq 0 then begin
  print,'ERROR in stelir_sort.pro: No files of that specification!!'
  retall
 endif

;get subdir name
 sls = strmid(dirname,strlen(dirname)-1,1)
 subdirs = strsplit(dirname,sls,/extract)
 subdir = subdirs[n_elements(subdirs)-1]

;get sort criterion
 case 1 of
  keyword_set(dark) or keyword_set(lin): pick = 'Yes'
  keyword_set(flat) or keyword_set(data): pick = 'No'
  else: pick = dialog_message(/question,'If data are DARK FRAMES or '+$
     'LINEARITY DATA (Sorted by Channel), click "Yes".  Otherwise, click '+$
     '"No" to Sort by Filter')
 endcase

 if pick eq 'Yes' then begin
  ndir = 2 & types = ['0','1'] & crit = 'CHANNEL'
 endif else begin
  ndir = 4 & types = ['J','H','K','L'] & crit = 'FILTER'
 endelse

;make new directories
 names = strarr(ndir)
 for i = 0,ndir-1 do begin
  names[i] = filepath(root_dir=dirname+'..',subdirectory=strmid(types[i],0,1)$
     +subdir,'')
  file_mkdir,names[i]
 endfor

 for i = 0,nf-1 do begin

;find the file name
  full = strsplit(f[i],sls,/extract)
  fname = full[n_elements(full)-1]

;read in the image
  print,'Reading '+f[i]
  fits_datmask,f[i],im,hdr,mask

;figure out where it goes
  rcrit = strupcase(strmid(sxpar(hdr,crit),0,1))
  k = where(types eq rcrit,nm)
  if nm ne 1 then begin
   print,'ERROR in stelir_sort.pro: Invalid '+crit+' entry!'
   retall
  endif

;write it to the correct directory
  print,'Writing '+names[k[0]]+fname
  fits_datmask,names[k[0]]+fname,im,hdr,mask,/write
 endfor

;eliminate unused directories
 for i = 0,ndir-1 do begin
  f = findfile(names[i]+'*?.?*',count=nf)
  if nf le 0 then begin
   print,'Deleting unused directory '+names[i]
   file_delete,names[i]
  endif
 endfor

 print,'Sorting of STELIRCam data in '+dirname+' completed...'
 print,'Check the results and then delete original data in '+dirname

end



