;..............................................................................
;
;   pv 
;                 of a data cube.
;
;                 Arguments
;
;                 cube = data cube array
;                 x = 1d array containing x coords as a function of pixel #
;                 y = 1d array containing y coords as a function of pixel #
;                 v = 1d array containing velocity as a function of pixel #
;		      (note uses fitaxes to produce these)
;                 xnum = size of map grid in x direction
;                 ynum = size of map grid in y direction
;                 vstart = starting velocity (upper left hand corner map)
;                 vdiff = velocity interval
;
;		  (the maps are integrated over intervals of vdiff starting
;		   with the velocity vstart)
;               
;		  keywords
;
;		  levels = 1-d array containing levels for contouring
;                 xrange = range in x plotted
;                 yrange = range in y plotted
;                 percent = if set, levels are percent of max value
;                         (note max is the maximum of ALL channel maps)
;                 order = order of x, y, and velocity axis in cube
;                         (default: "vxy")
;                 tv = if set, plot image using the low and high values
;		       of the levels array as the mininum and maximum
;                 nocontour = if set do not plot contours
;                 ps = if set sent to postscript device
;                 label = [x,y] position, if set the maps 
;		           will be labeled with the velocity interval at
;		           this location
;                 continue = used for overlaying contours onto plots in
;			     postscript mode.  First plot image setting
;			     /tv, /nocontours, /ps and /continue, then
;			     plot contours setting only /ps.
;
;
;----------------------------------------------------------------

pro pv,cube,x,y,v,pv_x,pv_y,xaxis=xaxis,yaxis=yaxis,vrange=vrange,$
       xrange=xrange,yrange=yrange,contour=contour,tv=tv,ps=ps,$
       levels=levels,order=order


;  set up initial values of margin size, cube size, and pixel increments
   margin = 0.1
   xsize_cube = n_elements(x)
   ysize_cube = n_elements(y)
   onexpix = abs(x[1]-x[0])
   oneypix = abs(y[1]-y[0])
 
;  set up initial value for display size, pixel size, and aspect ratio
   dpcm = 1.
   xdisp = !D.x_vsize
   ydisp = !D.y_vsize
   disp_asp = 1.*ydisp/xdisp  
   print,xdisp,ydisp,disp_asp

   pos_array = replicate(0.,4,xnum*ynum)
   map = replicate(0.,xsize_cube,ysize_cube,xnum*ynum)

; if keyword range isn't set, set the range to the maximum extent of the input data
   if keyword_set(xrange) then begin
      xsize = abs(xrange[1]-xrange[0])
   endif else begin
      xsize = n_elements(x)*onexpix
      xrange = [max(x)+onexpix/2.,min(x)-onexpix/2.]
   endelse  
   if keyword_set(yrange) then begin
      ysize = abs(yrange[1]-yrange[0])
   endif else begin
      ysize = n_elements(y)*oneypix
      yrange = [min(y)-oneypix/2.,max(y)+oneypix/2.]
   endelse
   aspect = (1.*ysize)/xsize
   print,"xrange =",xrange,"yrange =",yrange
 
; set up postscript if plot
   if keyword_set(ps) then begin
     dpcm=1000.                      ; dots/cm
     xpaper=21.6                     ; xsize of paper (cm)
     ypaper=27.9                     ; ysize of paper (cm)
     xmargin=2.5                     ; x-border (cm)
     ymargin=2.5                     ; y-border (cm)

     xdisp = 20
     ydisp = 20
     disp_asp = 1.
     xoffset=(ypaper-xdisp)/2.0              ; xoffset: center
     yoffset=(xpaper-ydisp*aspect)/2.0  
     set_plot,'ps',/copy
     device,/portrait,xoff=xoffset/4.,yoff=yoffset*2.,xsize=xpaper,ysize=ypaper
     device,/color,bits=8
     plot,x,y,xsty=5,ysty=5,/noerase,/nodata
;      loadct,34
;      loadct,3
   endif
	

; set size of plots and margin of plot grid
    if (disp_asp gt aspect*ynum/xnum) then begin
      xpsize = ((1.-margin)/xnum*xdisp)
      ypsize = (aspect*xpsize)
      marginx = (margin/2.*xdisp)
      marginy = (marginx*disp_asp)
      print,"X",xpsize,ypsize
    endif else begin
      ypsize = ((1.-margin)/ynum*ydisp)
      xpsize = (ypsize/aspect)
      marginy = (margin/2.*ydisp)
      marginx = (marginy/disp_asp)
      print,"Y",xpsize,ypsize
    endelse

; create a position array of the corners of each plot panel
   for j = 0,ynum-1 do begin
     for i = 0,xnum-1 do begin
       pos_array[*,i+j*xnum] = $
                [marginx+i*xpsize,marginy+(ynum-j-1)*ypsize,$
                 marginx+(i+1)*xpsize,marginy+(ynum-j)*ypsize]
     endfor
   endfor
   if keyword_set(ps) then pos_array = pos_array*dpcm
   print,pos_array

; integrate channel maps for each velocity interval
   for i = 0,xnum*ynum-1 do begin
     vel1 = vstart+i*vdiff
     vel2 = vstart+(i+1)*vdiff
     map[*,*,i] = int_slice(cube,x,y,v,vel1=vel1,vel2=vel2,order=order)
   endfor

; find maximum value of all the channel maps
   maxval = max(map)

   print,"maximum value of all maps =",maxval

; if the level keyword isn't set, then set levels of 30 to 100% of maximum value
   if NOT keyword_set (levels) then begin
     levels = maxval*(findgen(7)+3)/10.
   endif

; if precent level keyword is set, scale levels to precentage of max value
   if keyword_set (percent) then begin
     levels = levels*maxval/100.
   endif

; begin plotting of data
   for i = 0,xnum*ynum-1 do begin

; if tv keyword is set, interpolate data onto new grid and plot image
     if keyword_set(tv) then begin
       if NOT keyword_set(ps) then begin
         nxpix = fix(xpsize+.5)
         nypix = fix(ypsize+.5)
       endif else begin
         nxpix = 400
         nypix = 400*aspect
       endelse
       xnew = $
         findgen(nxpix)*(xrange[1]-xrange[0])/nxpix+xrange[0]
       ynew = $
         findgen(nypix)*(yrange[1]-yrange[0])/nypix+yrange[0]
       xinterp = interp_pix(xnew,x)
       yinterp = interp_pix(ynew,y)
       plotmap = interpolate(map[*,*,i],xinterp,yinterp,/grid,missing=0)
       tv,bytscl(plotmap,min=levels[0],max=levels[n_elements(levels)-1],$
          top = 254),$
         pos_array[0,i],pos_array[1,i],xsize=xpsize*dpcm,ysize=ypsize*dpcm
     endif

; plot contour map
     contour,map[*,*,i],x,y,xtickformat='(" ",a1)',ytickformat='(" ",a1)',$
             xstyle=1,ystyle=1,$
             position = pos_array[*,i],/noerase,$
             xrange = xrange, yrange = yrange,$
             levels = levels,/device,nodata=nocontours,$
             color=!d.table_size

; label each plot if keyword label is set
     if keyword_set(label) then begin
        xy = convert_coord(label[0],label[1],/data,/to_device)
       vel1 = vstart+i*vdiff
       vel2 = vstart+(i+1)*vdiff
        strlabel=strcompress(string(vel1)+","+string(vel2))
        xyouts,xy[0],xy[1],strlabel,color=!d.table_size,/device
     endif

   endfor

; plot contour map without data to set up axis values.
     contour,map[*,*,0],x,y,/nodata,$             
             xstyle=1,ystyle=1,$
             position = pos_array[*,0+(ynum-1)*xnum],/noerase,$
             xrange = xrange, yrange = yrange,/device,color=!d.table_size


; if making postscript file, close device and change plot back to 'x'
   if keyword_set(ps) and NOT keyword_set(continue) then begin
      device,/close
      set_plot,'x',/copy
   endif

   return
   end

   

