;+
; NAME:
;       PHOTMATCH4
;
; PURPOSE:
;       Match detections in multiple bands from PhotVis output files
;
; CALLING SEQUENCE:
;       result = PHOTMATCH()
;
; INPUTS:
;       NONE (GUI-based)
;
; OPTIONAL INPUTS:
;       NONE
;
; KEYWORDS:
;       TOL - distance tolerance in pixels for detection match
;       OFFSET - 2*(n_bands-1) element vector of offsets between source images
;                from the first band
;       FILTER - string array with filter names of data to be matched
;	NO_NEAR - set this switch to make pm_filter remove all detections with
;                 with other detections within tol pixels...
;
; OUTPUTS:
;       result is an n_bands+2 by n_matches element array 
;         [x,y,mag1,mag2(,mag3)]
;
; PROCEDURE CALLS:
;       NONE - no non-standard IDL subroutines are used...
;
; REVISION HISTORY:
;       07-01-02 - initial coding (RAG)
;	07-09-03 - added no_near keyword (RAG)
;       08-14-03 - bastardized to keep error info for fun new filtering (RAG)
;
;-


function photmatch4,tol=tol,offset=offset,filter=filter,no_near=no_near

 if not keyword_set(tol) then tol = 5.
 if not keyword_set(filter) then filter = ['J','H','K']
 nf = n_elements(filter)
 if not keyword_set(offset) then offset = fltarr(2*(nf-1))

 data = ptrarr(nf,/alloc)

 for i=0,nf-1 do begin
  name = dialog_pickfile(title='Select '+filter[i]+' band PV_DAT file',$
                             /must_exist)
  if name eq '' then begin
   print,'Cancelled...'
   retall
  endif
  restore,name
  if i gt 0 then begin
   pv_dat[0,*] = pv_dat[0,*] + offset[2*(i-1)]
   pv_dat[1,*] = pv_dat[1,*] + offset[1+2*(i-1)]
  endif
  if keyword_set(no_near) then pv_dat = pm_filter(temporary(pv_dat),tol)
  *data[i] = temporary(pv_dat)
 endfor

 sz = size(*data[0])
 k = 0

 for i = 0,sz[2]-1 do begin

  x = (*data[0])[0,i]
  y = (*data[0])[1,i]
  gdflg = 0
  xtmp = fltarr(nf) & ytmp = fltarr(nf) & mags = fltarr(nf) & magerrs = fltarr(nf)
  xtmp[0] = x  & ytmp[0] = y & mags[0] = (*data[0])[7,i] & magerrs[0] = (*data[0])[8,i]

  for j = 1,nf-1 do begin
   dist = sqrt(((*data[j])[0,*]-x)^2+((*data[j])[1,*]-y)^2)
   md = min(dist)
   if md lt tol then gdflg = gdflg+1
   w = (where(dist eq md))[0]
   xtmp[j] = (*data[j])[0,w]
   ytmp[j] = (*data[j])[1,w]
   mags[j] = (*data[j])[7,w]
   magerrs[j] = (*data[j])[8,w]
  endfor

  if gdflg eq nf-1 then begin

   x = mean(xtmp)
   y = mean(ytmp)

   if k eq 0 then out = [x,y,mags,magerrs] else begin
    temp = temporary(out)
    out = fltarr(2*nf+2,k+1,/nozero)
    out[*,0:k-1] = temporary(temp)
    out[*,k] = [x,y,mags,magerrs]
   endelse

   k = k+1
  endif
 endfor

 return,out

end

