;+
;   name:
;        sf_flat.pro
;
;   purpose:
;       a procedure which shows a slideshow of the fits images in a given
;       directory (of flat fields, hopefully) to the user.  On user approval,
;       the flats are dark subtracted, normalized to their respective medians,
;       and median_stacked.  The resulting final image is saved
;       in 'calib_images' as (date)_(filter)_flat.fit.
;
;   calling sequence:
;        result = sf_flat(filter=filter,dirname=dirname)
;
;   inputs:
;       NONE
;
;   keywords:
;       dirname = (string) subdirectory containing desired flats
;       filter = 1 character string with filter to match to
;
;   output:
;       a dark-subtracted, normalized, median flat, as a FITS file
;
;   called programs:
;       fits_datmask.pro
;       sxpar.pro
;       sxaddpar.pro
;       merge_mask.pro
;       bytecube.pro
;       mov3r.pro
;       median_stack.pro
;
;   history:
;       written by R. Gutermuth - 5-23-01
;       8-29-01 - added multi-platform capability and changed to fits_datmask
;                 usage to use individual masks for each image.  (RAG)
;       1-24-02 - rebuilt like sf_dark... (RAG)
;       2-12-02 - switched to standard sxpar.pro and sxaddpar.pro
;                 usage for header info and modification (RAG)
;
;-

function sf_flat,filter=filter,dirname=dirname

;get dirname if not provided
 if not keyword_set(dirname) then dirname=dialog_pickfile(/directory,$
    title='Select a directory with flat field images')
 if dirname eq '' then begin
  print,'ERROR in sf_flat.pro: No directory specified!'
  retall
 endif

;get flat field filenames
 f = findfile(dirname+'*?.?*',count=nf)
 if nf eq 0 then begin
  print,'ERROR in sf_flat.pro: No files of that specification!!'
  retall
 endif

;read in flats and merge their masks
 for i = 0,nf-1 do begin

  print,'Reading in '+f[i]+'...'
  fits_datmask,f[i],im,hdr,mask

  if i eq 0 then begin
   sim = size(im)

;mess around with the format of the date
   date = sxpar(hdr,'DATE-OBS')
   if rstrpos(date,'-') ne -1 then div = '-' else div = '/'
   date = strjoin(strsplit(date,div,/extract))

;get the other garbage
   channel = sxpar(hdr,'CHANNEL')

   if not keyword_set(filter) then filter = sxpar(hdr,'FILTER') else begin
    if sxpar(hdr,'FILTER') ne filter then begin
     print,'ERROR in sf_flat.pro: Filter entry mismatch!'
     retall
    endif
   endelse

   resdir = filepath(root_dir=dirname+'..',subdir=['..','calib_images'],'')
   darknm = findfile(resdir+'*_'+filter+'_dark.fit',count=nd)
   case 1 of
    nd gt 1: begin
     exlin = dialog_pickfile(path=resdir,filter='*_'+filter+'_dark.fit',$
        title='Select the desired dark frame',get_path=pickdir)

;boot 'em if they don't pick from the standard directory cuz it screws
;things up
     if pickdir ne resdir then begin
      print,'ERROR in sf_flat.pro: Picked from non-standard directory!'
      retall
     endif
    end
    nd eq 1: exlin = darknm[0]
    else: exlin = ''
   endcase

;get or make dark frame
   if exlin eq '' then begin
    dk = ptr_new(0)
    *dk = sf_dark(channel=channel)
    dark = (*dk).dark & dhdr = (*dk).hdr & dmask = (*dk).mask &ddate=(*dk).date
    print,'Writing '+resdir+ddate+'_'+filter+'_dark.fit'
    fits_datmask,/write,resdir+ddate+'_'+filter+'_dark.fit',dark,dhdr,dmask
    ptr_free,dk
   endif else begin
    print,'Reading '+exlin
    fits_datmask,exlin,dark,dhdr,dmask
   endelse

   flats = ptr_new(make_array(sim[1],sim[2],nf,type=sim[sim[0]+1],/nozero))
   fmask = merge_mask(temporary(mask),temporary(dmask))
  endif else begin
   fmask = merge_mask(mask,temporary(fmask))
  endelse

  im = temporary(im) - dark
  (*flats)[*,*,i] = im / median(im)

 endfor

;make slideshow
 fltb = bytecube(*flats,mask=fmask)

 if fix(fltb[0]) eq -1 then begin
  print,'ERROR in sf_flat.pro:  Something broke in bytecube.pro!!'
  retall
 endif else begin
  window,/free,xs=sim[1]+75,ys=sim[2]+75,title="Current Flat Fields"
  mov3r,fltb
  huh = dialog_message(/question,'Median and save this flat field?')
  wdelete,!d.window
  if huh eq 'Yes' then begin
   result = median_stack(*flats)
   sxaddpar,hdr,'HISTORY','median of '+strtrim(sim[3],2)+' dark-subtracted '$
      +'images'
   fits_datmask,/write,resdir+date+'_'+filter+'_flat.fit',result,hdr,fmask
   print,resdir+date+'_'+filter+'_flat.fit written...'
  endif
 endelse

;dump flats from memory
 ptr_free,flats
 flat = {flat:result,hdr:hdr,mask:fmask,date:date}
 return,flat

end
