;+
;   name:
;        stelir_proc.pro
;
;   purpose:
;        a procedure which processes the fits images in a given directory
;	 and saves the results in 'processed' and makes and saves the 
;	 skyframes in 'skyframes'.  
;  	 (Made for Mt. Hopkins STELIRCAM data)
;
;   calling sequence:
;        stelir_proc,blk=blk
;
;   inputs:
;	(NONE) all through gui prompts
;
;   keywords:
;	blk = (integer) set to number of files per file block (default=18)
;		(can be determined from observing log)
;
;   side effects:
;   	makes 'processed', a new subdir of dirname
;	and 'skyframes'
;
;   output:
;       a processed image for every data image it finds
;	and the skyframes created
;
;   called programs:
;       fits_datmask.pro
;       merge_mask.pro
;	fits_str.pro (Rob's version)
;	fits_add.pro (Rob's version)
;	rob_pf.pro
;	makesky.pro (Rob's version)
;
;   history:
;       written by R. Gutermuth - 6-1-01
;	last modified - 6-20-01
;
;-

pro stelir_proc,blk=blk

;multi-platform capability
 if strlowcase(!version.os_family) eq 'windows' then sls = '\' else sls = '/'

;Read in flat fields
 fdirname1 = rob_pf(/directory,title='Select a Directory with '+$
	'H and K Flat Fields')

 print,fdirname1
 print,'Reading H and K Flat Fields...'

;initialize pointers
 flats = replicate(ptr_new(1),4)
 flat_masks = replicate(ptr_new(0),4)

 if file_test(fdirname1+'Hskyflat.fit') then begin
  fits_datmask,fdirname1+'Hskyflat.fit',*flats[1],hdr,*flat_masks[1]
  fits_datmask,fdirname1+'Kskyflat.fit',*flats[2],hdr,*flat_masks[2]
 endif else print,'No H or K flats.  Skipping...'

 fdirname2 = rob_pf(/directory,title='Select a Directory with '+$
	'J and L Flat Fields')

 print,fdirname2
 print,'Reading J and L Flat Fields...'

 if file_test(fdirname2+'Jskyflat.fit') then begin
  fits_datmask,fdirname2+'Jskyflat.fit',*flats[0],hdr,*flat_masks[0]
  fits_datmask,fdirname2+'Lskyflat.fit',*flats[3],hdr,*flat_masks[3]
 endif else print,'No J or L flats.  Skipping...'

;get image filenames
 dirname = rob_pf(/directory,title='Select a Directory with Data to Process')
 f = findfile(dirname+'*.*',count=nf)

 if nf eq 0 then begin
  print,'ERROR in stelir_proc.pro: No files of that specification!!'
  return
 endif

;read in images in blocks of 18 files (to make sky frames) and process them.
 if not keyword_set(blk) then blk = 18

;initialize arrays and pointers
 ims = make_array(sd[1],sd[2],blk,type=sd[sd[0]+1],/nozero)
 im_masks = replicate(ptr_new(0),blk)
 outnames = strarr(blk)
 chs = intarr(blk,/nozero)

;make receiving directories
 file_mkdir,dirname+'processed'
 file_mkdir,dirname+'skyframes'

 i=0

 while i lt nf do begin
 
  for j = 0,blk-1 do begin

;read in images and masks
   print,'Reading in '+f[i+j]+'...'
   fits_datmask,f[i+j],im,hdr,mask

;initialize header array
   if i eq 0 and j eq 0 then begin
    sh = size(hdr)
    hdrs = strarr(sh[1],blk)
   endif

;make output file name and keep in outname array
   dl = strlen(f[i+j])
   slsh = rstrpos(f[i+j],sls)
   fnm = strmid(f[i+j],slsh+1,dl-slsh-1)
   outnames[j] = dirname+'processed'+sls+fnm
 
   channel = fits_str('CHANNEL',hdr)
   ch = fix(strmid(channel,1,1))
   filter = strmid(fits_str('FILTER',hdr),1,1)

;check which filter...  this being for NIR, only J,H,K, and L are supported!
   case filter of
    'J': fin = 0
    'H': fin = 1
    'K': fin = 2
    'L': fin = 3
    else: begin
           print,'ERROR in stelir_proc.pro: Invalid FILTER entry'+$
              ' in header of image '+strtrim(i,2)+'!!!'
           print,filter
           return
          end
   endcase
 
;merge image mask with proper flat mask and store
   *im_masks[j] = merge_mask(mask,*flat_masks[fin])

;make mask image, then flat image and apply mask
   maskim = im * 0 + 1
   maskim[*im_masks[j]] = 0
   im = temporary(im) * maskim / (*flats[fin])
 
;store it all
   ims[*,*,j] = temporary(im)
   hdrs[*,j] = temporary(hdr)
   chs[j] = temporary(ch)

;get angry if this isn't STELIRCam data
   if ch ne 0 and ch ne 1 then begin
    print,'ERROR in stelir_proc.pro: Invalid CHANNEL entry'+$
      ' in header of image '+strtrim(i,2)+'!!!'
    print,channel
    return
   endif

  endfor

;free flat pointers 
  ptr_free,flats,flat_masks

;make skyframes using makesky.pro and save results in 'skyframes'

;initialize an array and pointers
  sky = replicate(ptr_new(0),2)
  sf_masks = replicate(ptr_new(0),2)
  medsky = fltarr(2)

;check which images are from which channels
  for l = 0,1 do begin
   w0 = where(chs eq l,c0)
   if c0 ne 0 then begin

;merge the masks for the skyframe and associated images
    *sf_masks[l] = *im_masks[w0[0]]
    for k = 1,c0-1 do $
       *sf_masks[l]=merge_mask(temporary(*sf_masks[l]),*im_masks[w0[k]])

;make the skyframe and write in 'skyframes'
    *sky[l] = makesky(ims[*,*,w0],f[w0+i])
    medsky[l] = median(*sky[l])
    hdr = fits_add("COMMENT = 'flatted sky frame; median of "+strtrim(c0,2)+$
       " images' /",hdrs[*,w0[0]])
    fits_datmask,/write,dirname+'skyframes'+sls+strtrim(l,2)+'skyframe'+$
       strtrim(i/blk,2)+'.fit',*sky[l],hdr,*sf_masks[l]
   endif
  endfor
  
;subtract skyframes, add median of skyframe, save final images in 'processed'
  for j = 0,blk-1 do begin
   im = ims[*,*,j] - *sky[chs[j]] + medsky[chs[j]]
   hdr = fits_add("COMMENT = 'sky frame subtracted; flat fielded' ",hdrs[*,j])
   print,'Writing '+outnames[j]+' ...'
   fits_datmask,/write,outnames[j],im,hdr,*sf_masks[chs[j]]
  endfor
 
;switch to next block of files
  i = i + blk
 endwhile

;free the rest of the pointers and end
 ptr_free,im_masks,sky,sf_masks
 print,'Sky frame subtraction and flat fielding completed!'
end


