;+
; NAME:
;       ADDMASK
;
; PURPOSE:
;       returns a long integer vector of pixel indeces for addition to IR 
;       pipeline mask vectors.
;
; CALLING SEQUENCE:
;       result = ADDMASK(x, y [, hdr])
;
; INPUTS:
;       x - x coordinate (single entry for now)
;       y - y coordinate (single entry for now)
;
; OPTIONAL INPUTS:
;       hdr - string array (FITS standard header), used to reference desired
;             image dimensions...  If unspecified, 256 x 256 is assumed.
;
; KEYWORDS:
;       RAD - if specified, the indeces of all positions within rad pixels of 
;             [x,y] including [x,y] are returned.
;       NO_VECT - set to return mask as a 2D binary image (default is to
;                 return vector of bad pixels)
;
; PROCEDURE CALLS:
;       sxpar()
;
; REVISION HISTORY:
;       6/02 - initial coding (RAG)
;       1/24/03 - cleaned up and optimized some stuff, entered into pipeline. 
;                 (RAG)
;       1/27/03 - hopefully fixed nappy bug in circle masking stuff...  (RAG)
;       10/28/03 - hopefully fixed a second nappy bug in the circle masking 
;                 stuff regarding float input x and y...  (RAG)
;
;-

function addmask,x,y,hdr,rad=rad

 if n_params() eq 3 then begin
  xs = sxpar(hdr,'NAXIS1')
  ys = sxpar(hdr,'NAXIS2')
 endif else begin
  print,'No header supplied...  Assuming 256x256'
  xs = 256l
  ys = 256l
 endelse

 if x lt 0 or x ge xs or y lt 0 or y ge ys then begin
  print,'ERROR in addmask.pro: Position off array!'
  retall
 endif

 if keyword_set(rad) then begin

  rad = float(rad)

  if rad gt max([xs,ys])*sqrt(2.) then return,lindgen(xs*ys)

  rds = long(4*rad)
  maxlong = 2l^31 - 1

  rdist = fltarr(rds,rds)
  index = lonarr(rds,rds)+maxlong

  offset = [fix([x,y]) - rds/2, fix([x,y]) + rds/2 - 1/(2-(rds mod 2))]

  minx = 0l - (offset[0] < 0)
  miny = 0l - (offset[1] < 0)
  maxx = rds-1 - ((offset[2] - xs) > 0)
  maxy = rds-1 - ((offset[3] - ys) > 0)

  for j = miny,maxy do index[minx:maxx,j] = (j+offset[1]>0)*xs+lindgen(maxx-minx+1)+(offset[0]>0)

  for j=0,rds-1 do for i=0,rds-1 do begin
   xin = index[i,j] mod xs & yin = index[i,j] / xs
   rdist[i,j]=sqrt((float(x)-xin)^2+(float(y)-yin)^2)
  endfor

  wtmp = where(rdist le rad)
  w = index[wtmp] 

 endif else w = [long(xs*y+x)]

 return,w

end
