;+
;   NAME:
;     FDS_MASTER
;
;   PURPOSE:
;     This function reads in a series of images and combines them in any of
;     a number of ways, as specified by the user, and returns the resulting
;     image, header, and mask as an anonymous structure.  Both median and
;     sigma-chopped mean image combination methods are available, along
;     with the ability to subtract an image from each prior to combination, and
;     the ability to normalize the resulting image to it's median value.
;
;   CALLING SEQUENCE:
;     result = FDS_MASTER()
;
;   INPUTS:
;     (NONE) all inputs are either GUI-based or keyword-based
;
;   KEYWORDS:
;       INFILES = string array of FITS filenames for processing image
;                 (GUI prompt if not specified)
;       OUTNAME = string containing filename to save resulting FITS image to
;                 (GUI prompt if not specified, can disable by specifying 'NA')
;       SUBFILE = string containing name of an existing FITS file 
;                 (GUI prompt if set but invalid, can disable by specifying
;                 'NA')
;       MEDNORM = set to enable median normalization of final result (GUI prompt
;                 if not set, can disable by specifying -2)
;       SCMSTACK = set to use sigma-chopped mean image combination
;       MEDSTACK = set to use median image combination (if neither scmstack or
;                  medstack are set, or both are set, GUI prompt settles things)
;       SKY = set to tell the program the result is intended to be a sky frame.
;             if a skyframe, no median normalization, no dark/sky subtraction,
;             and we offset by median of each image instead of divide when 
;             making median stack. 
;             (GUI prompt if not set, can say "not a sky" by specifying -2)
;       CHOPSIG = float, number of sigma outside which a pixel gets masked out
;                 (default is 3.0)
;       NO_PLOT = set to disable the status plots
;       USELOG = boolean (gives GUI prompt) or filename of fds_master log
;                to (re)run
;       SAVELOG = filename to save logfile as
;       NO_SAVELOG = set to not be prompted to save logfile (gets set
;                    if uselog was used and a log was read in)
;       REFFILE = string containing name of an existing FITS file that is used 
;                 to override the making of the reference image...  Should be 
;                 median normed, median stack of large number of images for 
;                 accurate reference for sigma-chopped mean process (GUI prompt
;                 if set but invalid)
;
;   OUTPUTS:
;       result is an anonymous structure of the format:
;        name: string containing filename (can be null string if unsaved)
;        image: 2D image
;        header: string array with FITS format header
;        mask: mask vector
;       these are also written to disk as a FITS file
;       (with mask vector as extension).
;
;   CALLED PROCEDURES:
;       fits_datmask, hlp, merge_mask(), median_stack2(), sxaddpar(), 
;       logcheck(), robomean
;
;   MODIFICATION HISTORY:
;     5-15-02: completed and integrated into pipeline (RAG)
;     1-22-03: merged Mark Buie's avgclip.pro method with our pipeline mask 
;              management to yield a superior sigma-chopped result.  Sadly, the 
;              memory economy of this program has been lost in the process, so 
;              be prepared to go to virtual memory (and thus have a looooooong 
;              wait) if you lack the computing muscle to handle the amount of 
;              data you feed in. (RAG)
;     1-24-03: fixed yucky error in application of method for sky frames.
;     2-12-03: tweaked display stuff, as well as filtering full images on their
;              relative median values...  (RAG)
;     3-13-03: added date/time logging to HISTORY comment in FITS header (RAG)
;     5-20-03: added reffile, for overriding the usual reference image with an 
;              external image...  Use at your own risk!  (RAG)
;
;-


function fds_master,mednorm=mednorm,scmstack=scmstack,medstack=medstack,$
                    chopsig=chopsig,infiles=infiles,subfile=subfile,$
                    outname=outname,no_plot=no_plot,sky=sky,uselog=uselog,$
                    no_savelog=no_savelog,savelog=savelog,reffile=reffile

;****** begin user input section

;restore saved inputs if desired
 huh = logcheck(uselog)
 if huh then begin
  restore,uselog & no_savelog = 1
 endif

;get main image names if not supplied...
;two copies of filenames cuz I'm too lazy to change the code...  ;)
 if keyword_set(infiles) then f = infiles else begin
  f = dialog_pickfile(/multiple,/must_exist,$
                      title='Select files to go into processing image')
  infiles = f
 endelse

;check for filenames
 nf = n_elements(f)
 if f[0] eq '' then begin
  print,'ERROR in fds_master.pro: Missing file selection(s)!'
  retall
 endif

;check for desire to make a sky frame...  don't want skyframe? set sky = -2
 if not keyword_set(sky) then begin
  yup1 = dialog_message(/ques,'Is this intended to be a Sky Frame?')
 endif else begin
  if sky then yup1 = 'Yes' else yup1 = 'No'
 endelse

;if this is a sky frame, no normalizing and no dark/sky subtraction allowed
 if yup1 eq 'No' then begin
  sky = -2

;deal with median normalization...  don't want it?  set mednorm=-2
  if not keyword_set(mednorm) then begin
   yup1 = dialog_message(/ques,'Normalize result to its median value?')
   if yup1 eq 'Yes' then mednorm = 1 else mednorm = -2
  endif
  if mednorm then medtxt = 'Median-normed, ' else medtxt = ''

;check for desire to subtract dark/sky frame
  if not keyword_set(subfile) then begin
   subfile = 'NA'
   yup1 = dialog_message(/ques,'Do you want to subtract an existing'+$
                        ' sky or dark frame?')
  endif else begin
   if subfile ne 'NA' then begin
    ssub = size(subfile)
    if ssub[ssub[0]+1] ne 7 then yup1 = 'Yes' else begin
     if not file_test(subfile) then begin
      print,'ERROR in fd_master.pro: '+subfile+' does not exist!'
      yup1 = 'Yes'
     endif else yup1 = 'No'
    endelse
   endif else yup1 = 'No'
  endelse
  if yup1 eq 'Yes' then subfile = dialog_pickfile(/must_exist,$
     title='Select sky or dark frame to subtract')
  if subfile eq '' then begin
   subfile = 'NA'
   print,'Cancelling dark or sky frame subtraction...'
  endif
 endif else begin
  sky = 1
  mednorm = -2
  subfile = 'NA'
  medtxt = ''
 endelse

;for whatever reason, report whether or not an image will be subtracted
;don't want to subtract an image?  set subfile='NA'
 if subfile eq 'NA' then begin
  darktxt = ''
  print,'Not subtracting dark or sky frame...'
  dark = 0.
  dmask = -1
 endif else begin
  darktxt = 'dark-(or sky-)subtracted '
  print,'Reading '+subfile
  fits_datmask,subfile,dark,dhdr,dmask
 endelse

 if not keyword_set(no_plot) then no_plot = -2

;chopsig is number of std deviations to keep (from median)
 if not keyword_set(chopsig) then chopsig = 3.

;get desired merging method if not specified
 if not (keyword_set(scmstack) or keyword_set(medstack)) or $
    (keyword_set(scmstack) and keyword_set(medstack)) then begin
  yup1 = dialog_message(/question,'Use median stack method? (Default is '+$
     strtrim(chopsig,2)+' sigma-chopped mean)')
  if yup1 eq 'Yes' then begin
   medstack = 1 & scmstack = 0 & reffile = 'NA'
  endif else begin
   scmstack = 1 & medstack = 0
  endelse
 endif else begin
  if not keyword_set(medstack) then medstack = 0
  if not keyword_set(scmstack) then begin
   scmstack = 0
   reffile = 'NA'
  endif
 endelse

;override reference image creation for sigma-chopped mean by using existing FITS file
 if not keyword_set(reffile) then reffile='NA'

 if string(reffile) ne 'NA' then begin
  ssub = size(reffile)
  if ssub[ssub[0]+1] ne 7 then yup1 = 'Yes' else begin
   if not file_test(reffile) then begin
    print,'ERROR in fds_master.pro: '+reffile+' does not exist!'
    yup1 = 'Yes'
   endif else yup1 = 'No'
  endelse
 endif else yup1 = 'No'
 if yup1 eq 'Yes' then reffile = dialog_pickfile(/must_exist,$
    title='Select FITS file for reference image override')
 if reffile eq '' then begin
  reffile = 'NA'
  print,'Cancelling reference image override...'
 endif

 if reffile ne 'NA' then begin
  print,'Reading '+reffile
  fits_datmask,reffile,refimage,refhdr,refmask
 endif

;get outname if not provided
 if not keyword_set(outname) then begin
  huh = 'No'
  while huh ne 'Yes' do begin
   outname = dialog_pickfile(title='Save result as?')
   if file_test(outname) then huh = dialog_message(/question,'File '+outname+$
      ' exists...  Overwrite?') else huh = 'Yes'
   if outname eq '' then begin
    print,'Cancel: Not writing result to disk...'
    huh = 'Yes'
   endif
  endwhile
 endif

;save log or not?
 loghuh = logcheck(savelog,/save,no_save=no_savelog)

;write log if it's called for
 if loghuh then save,filename=savelog,infiles,outname,subfile,mednorm,chopsig,$
    scmstack,medstack,no_plot,sky,reffile

;****** end user input section

;first, get median of first few images of im - "dark"

;if using median, this is the only step
 if scmstack then begin
  print,'Phase 1: Get median '+darktxt+'image of the dataset...'
 endif else begin
  print,'Computing median '+darktxt+'image of dataset...'
 endelse

 done = nf-1

 for i = 0,done do begin
  print,'Reading '+f[i]
  fits_datmask,f[i],im,hdr,inmask

  im = temporary(im) - dark
  
;merge incoming data mask with dark frame mask
  inmask = merge_mask(inmask,dmask,nbad)
  if nbad gt 0 then im[inmask] = 0.

  wneg = where(im lt 0,nneg)
  if nneg gt 0 then begin
   inmask = merge_mask(inmask,wneg,nbad)
   im[wneg] = 0.
  endif

;merge incoming mask with overall mask (in agreement mode, as transients
;should get ignored through median)
  if i eq 0 then dinmask=inmask else dinmask=merge_mask(dinmask,inmask,/agree)

;initialize arrays needed
  if i eq 0 then begin
   sz = size(im)
   mask_ptrs = ptrarr(nf,/alloc)
   cube = make_array(done+1,sz[1],sz[2],type=sz[sz[0]+1],/nozero)
   med = fltarr(done+1,/nozero)
   numbad = lonarr(done+1,/nozero)
  endif

  *mask_ptrs[i]=inmask

;scale each image by it's own median value
  med[i] = median(im)
  numbad[i] = nbad
  cube[i,*,*] = temporary(im) / med[i]
 endfor

;dump dark frame from memory
 delvarx,dark

;get median image of cube
 md = median(med)
 if reffile ne 'NA' then begin
  print,'Reference image override, median stack skipped'
  avg = refimage
 endif else avg = median_stack2(cube)

;display median image
 if not no_plot then begin
  window,13,xs=512,ys=512,title = 'fds_master.pro'
  tvscl,congrid(avg,256,256)>.8<1.2
 endif
 hlp,avg

;jump to the end and write the file if median was chosen.
;otherwise, move on to computing sigma map.
 if medstack then begin
  final = temporary(avg)
  if sky then final = (temporary(final) * md) - md
  outmask = temporary(dinmask)
  sxaddpar,hdr,'HISTORY',systime(/utc)+' UTC: '+medtxt+'median of '+$
     strtrim(nf,2)+' '+darktxt+'images'
 endif else begin
  print,'Phase 1 Completed!'

;second, get std deviation image, now just sqrt of median image.
;From here on out, assume sigma chopped mean...
  print,'Phase 2: Get standard deviation image of '+darktxt+'images'

  std = sqrt(avg>0)

  if not no_plot then tvscl,congrid(std,256,256)>.8<1.2,256,0,/dev
  hlp,std

  print,'Phase 2 Completed!'

;third, chop high sig values, keep mask tally, and make final
;sigma-chopped mean
  print,'Phase 3: Mask values beyond +/- '+strtrim(chopsig,2)+$
        ' standard deviations and recompute final "mean" image'

  badvect = lonarr(nf)
  sigs = fltarr(nf)
  robomean,med,bad=badvect,2.5,.5

  for i = 0,nf-1 do begin
   print,'Filtering '+f[i]
   inmask = *mask_ptrs[i]
   im = reform(cube[i,*,*])
   im[inmask]=-10000
   wgood = where(im ne -10000,ngood) 
   im[inmask]=0. 
   if not badvect[i] and ngood gt 0 then begin
    check = (im-avg) / (std>0.000001)

;replicate skysclim.pro with mask included
    idx=randomu(seed,min([1000,n_elements(im)-numbad[i]]))*(n_elements(im)-numbad[i]-1)
    robomean,check[wgood[idx]],2.0,0.5,meanval,dummy,rsig
;and pray that this works...

    check = temporary(abs(check))/rsig
;    check = temporary(check)/rsig
    print,i,rsig
    *mask_ptrs[i] = merge_mask(where(check gt chopsig),temporary(inmask),nbad)
    numbad[i] = nbad
    sigs[i] = rsig
   endif else begin
    badvect[i] = 1
    numbad[i] = sz[1]*sz[2]
    *mask_ptrs[i] = -1
   endelse
  endfor

;stop

  robomean,sigs,bad=badvect,3.0,.5

  for i = 0,nf-1 do begin
   if i eq 0 then begin
    final = im * 0.
    norm = long(final) + nf
   endif 

   if badvect[i] then begin
    print,'Rejecting '+f[i]
    im = 0
    norm = norm-1
   endif else begin
    im = reform(cube[i,*,*])
    if sky then im = med[i] * temporary(im) - med[i] else im = md * temporary(im)
    ;if sky then im = med[i] * temporary(im) - med[i] else im = med[i] * temporary(im)
    if numbad[i] gt 0 then begin
     im[*mask_ptrs[i]] = 0.
     norm[*mask_ptrs[i]] = norm[*mask_ptrs[i]] - 1
    endif
   endelse

    final = temporary(final) + temporary(im)
  endfor

;don't need these anymore
  delvarx,avg,std,check,cube

;any positions that were all excluded are bad pixels...  zero them out,
;then set norm at that position to 1 to keep from dividing by zero.
  w = where(norm le 0,nbad)
  if nbad gt 0 then begin
   norm[w] = 1
   final[w] = 0.
   outmask = w
  endif else outmask = -1

;display norm
  if not no_plot then tvscl,congrid(norm,256,256)>1<nf,0,256,/dev
  hlp,norm

;get final mean!  WOO HOO!!
  final = temporary(final) / temporary(norm)

  print,'Phase 3 Completed!'
  sxaddpar,hdr,'HISTORY',systime(/utc)+' UTC: '+medtxt+strtrim(chopsig,2)+$
     ' sigma-chopped mean of '+strtrim(nf,2)+' '+darktxt+'images'
 endelse

 if sky then final = temporary(final) + md

 scl = median(final)

;normalize image to median if desired
 if mednorm then begin
  final = temporary(final) / scl
  scl = 1
 endif

;display final result, whichever method was used
 if not no_plot then tvscl,congrid(final,256,256)>.8*scl<1.2*scl,256,256,/dev
 hlp,final

;write results with fits_datmask
 if outname ne '' and outname ne 'NA' then begin
  print,'Writing '+outname
  fits_datmask,/write,outname,final,hdr,outmask
 endif else print,'Not saving result.'

;return output as an anonymous structure
 out = {name:outname,image:final,header:hdr,mask:outmask}
 return,out

end
