;+
;   name:
;        rob_assemble.pro
;
;   purpose:
;       this function is a convenient little front-end for assemble.pro
;       made to allow me to feed that function the images in a dataset
;       in such an order as to allow coadding and true 'mosaicing' to
;       occur in the same call, but with added automation for coadds.
;
;   calling sequence:
;       result = rob_assemble(npm=npm,order=order)
;
;   inputs:
;  	NONE (all inputs are through GUI's)
;
;   keywords:
;       npm = (integer) number of images per mosaic (default is 9)
;       order = (npm element vector) specifies the zero-based indeces of the
;               order of images to be mosaiced (default is 0 to npm-1 in order)
;	no_coadd = switch to make a rob_reg_gui spawn for every image.
;       no_mos = switch to sort coadded sections by dithers instead of by map 
;                sections.  (Sorry, I explained that poorly...  if you don't 
;                know, don't use it or ask Rob!)
;
;   output:
;       result = structure including the 'mosaic', mask for the mosaic
;                ('master'), xoffsets ('xoff'), yoffsets ('yoff')
;
;   called programs:
;       rob_fd, fits_read, fits_datmask, sxpar(), assemble()
;
;   history:
;       written 6-15-01: R. Gutermuth
;       6-20-01: changed order of final mosaic call so that we start
;                with the center frame and hop around from there (RAG)
;       6-21-01: changed order again...  start with center frame and 
;                proceed counterclockwise from right side (RAG)
;       7-2-01:  added multi-platform capability (RAG)
;       9-5-01:  made changes to accommodate fits_datmask usage (RAG)
;       6-6-02: eliminated much of the crap commented code, plus updated
;               user interface to be somewhat similar to the current pipeline
;               "standard" (RAG)
;       6-10-02: made some changes to make the end result more ATV friendly(RAG)
;       10-31-02: reenabled ditching of leftover images that don't make a full 
;                 map...  Happy Halloween!! (RAG) 
;       3-10-03: added no_coadd switch to bypass automated coadding...  Also
;                tweaked behavior of dumping bad images so that one can dump 
;                submosaic master images...  Also added autoscaling of displays
;                for big images... (RAG)
;       3-13-03: added date/time logging in HISTORY comment to FITS header (RAG)
;       7-4-03: added no_mos for fun...  sorts files differently, for when 
;               coadded files are adjacent to each other.  Also fixed a Windows 
;               compatibility bug... And made the image dumping display use an 
;               image's mask before displaying. (RAG) 
;       10-22-03: bunch of improvements for later remosaicing. (RAG)
;       10-29-03: added check for IRAC channel ID. (RAG)
;       4-5-04: fixed SQIID filter read...  We'll see what other creative names 
;               for FILTER in FITS headers folks can come up with for other 
;               instruments.  Also, fixed bug that kept ind from being saved in 
;               rob_assemble.sav. (RAG)
;
;-


function rob_assemble,npm=npm,order=order,no_coadd=no_coadd,no_mos=no_mos,$
                      infiles=infiles

;get main image names if not supplied
 if keyword_set(infiles) then f = infiles else begin
  infiles = dialog_pickfile(/multiple,/must_exist,$
                     title='Select all files to go into mosaic')
  f = infiles
 endelse
 
;check for filenames
 nf = n_elements(f)
 if f[0] eq '' then begin
  print,'ERROR in rob_assemble.pro: Missing file selection(s)!'
  retall
 endif

;multi-platform capability
 if strlowcase(!version.os_family) eq 'windows' then begin
  sls = '\'
  sls2 = ''
 endif else begin
  sls = '/'
  if strmid(f[0],0,1) eq sls then sls2 = sls else sls2 = ''
 endelse

 temp = strsplit(f[0],sls,/extract)
 ntmp = n_elements(temp)
 if ntmp gt 1 then sdir = sls2+strjoin(temp[0:n_elements(temp)-2],sls)+sls $
    else sdir = '.'+sls

;get filter from master header
 fits_read,infiles[0],im,hdr,/header_only
 filter = strmid(sxpar(hdr,'FILTER',count=np),0,1)
 if np eq 0 then filter = strmid(strtrim(sxpar(hdr,'CHANNEL',count=np),2),0,1)
 if np eq 0 then filter = strmid(strtrim(sxpar(hdr,'ACHANID',count=np),2),0,1)
 if np eq 0 then filter = ''

;sort image names by subsections of mosaic for coadding
 if not keyword_set(npm) then npm = 9
 nmos = fix(nf/npm)
 if nf ne nmos*npm then begin
  huh = dialog_message(strtrim(nf,2)+' images not integer number of '+strtrim(npm,2)+'-piece maps!  Cancel?',/ques)
  if huh eq 'Yes' then retall else begin
   print,'Last '+strtrim(nf-nmos*npm,2)+' images ignored...  Expect mis-collated sections!'
   nf = nmos*npm
   f = f[0:nmos*npm-1]
  endelse
 endif

;make mosaic directory
 file_mkdir,sdir+filter+'mosaic'

; openw,one,sdir+filter+'mosaic'+sls+'filenames.tmp',/get
; for i=0,nf-1 do printf,one,f[i]
; free_lun,one
 
;arrange filenames into 2D array
 if keyword_set(no_mos) then begin 
  ftmp = f
  f = strarr(nmos,npm)
  for i = 0,nmos-1 do f[i,*] = ftmp[0+i*npm:npm-1+i*npm]
  npmtmp = npm
  npm = nmos
  nmos = npmtmp
 endif else begin
  f = reform(f,npm,nmos,/overwrite)
 endelse

;open submosaic name files
 openw,one,sdir+filter+'mosaic'+sls+'mosaics.tmp',/get

 if not keyword_set(order) then ind = indgen(npm) else ind = order

;save this stuff for later remosaicing
 save,file=sdir+filter+'mosaic'+sls+'rob_assemble.sav',f,ind

 for l = 0,npm-1 do begin

  i = ind[l]

;make appropriate files of filenames and hand to assemble.pro
  moslist = sdir+filter+'mosaic'+sls+'sm_list'+strtrim(i,2)+'.tmp'
  openw,three,moslist,/get
 
  print,f[i,0]
  for j = 1,nmos-1 do begin
   printf,three,f[i,j]
   print,f[i,j]
  endfor
  free_lun,three
  currentmaster=f[i,0]
  nnow = nmos

;call assemble using coadd switch
  ahuh = 'No'
  while ahuh ne 'Yes' do begin

   if ahuh eq 'No' then begin
    bhuh = dialog_message(/question,'Do you want to dump any bad images?')
    if bhuh eq 'Yes' then begin
     newmaster = 0
     tnm = loadlist(moslist)
     nm = [currentmaster,tnm]	
     openw,three,moslist,/get
     openw,five,sdir+filter+'mosaic'+sls+'rejects.txt',/get,/append
     snm = size(nm)
;     nnm = make_array(size=snm)
     for k = 0,snm[1]-1 do begin
;      fits_read,nm[k],im
      fits_datmask,nm[k],im,bullhdr,bullmask
      im = im/median(im)
      bmsz = size(bullmask)
      if bmsz[0] eq 2 then bullmask = where(bullmask lt .1)
      if bullmask[0] ne -1 then im[bullmask] = 1.
      if k eq 0 then begin
       sim = size(im)
       zflg = max(sim[1:2])/512
       sim[1:2] = sim[1:2]/zflg
       immo = moment(im,sdev=imsd)
       window,8,xs=sim[1],ys=sim[2]+10
      endif
      erase
      tv,bytscl(congrid(im,sim[1],sim[2]),min=immo[0]-2*imsd,max=immo[0]+2*imsd)
      xyouts,fix(sim[1]/2),sim[2],strmid(nm[k],rstrpos(nm[k],sls)+1,$
         strlen(nm[k])-rstrpos(nm[k],sls)-1),align=.5,/dev
      chuh = dialog_message(/question,'Keep it?')
      if chuh eq 'Yes' then begin
       if newmaster then begin
        currentmaster = nm[k]
        newmaster = 0
       endif else if k gt 0 then printf,three,nm[k]
      endif else begin
       if k eq 0 then newmaster = 1
       printf,five,nm[k]
       nnow = nnow - 1
      endelse
     endfor
     free_lun,three,five
    endif
   endif

   result=assemble(currentmaster,moslist,coadd=1-keyword_set(no_coadd),scl=scl,zflg=zflg)

   ahuh = dialog_message(/question,'Did things register properly?')

  endwhile

;save results and names to file
;note: we need to keep mask _images_ now that the masks are more than
;      zeros and ones
  mosnm = sdir+filter+'mosaic'+sls+'submosaic'+strtrim(i,2)+'.fits'
  print,mosnm
  if l gt 0 then begin
   printf,one,mosnm
  endif else begin
   mmosnm = mosnm
  endelse
  offsets = [result.xoff,result.yoff]
  moshdr = hdr
  sxaddpar,moshdr,'HISTORY',systime(/utc)+' UTC: mosaic of '+strtrim(nnow,2)$
     +' images'
  fits_datmask,/write,mosnm,result.mosaic,moshdr,result.master
  openw,three,sdir+filter+'mosaic'+sls+'sm_offsets'+strtrim(i,2)+'_f_'+$
     strtrim(n_elements(offsets),2),/get
  writeu,three,offsets
  free_lun,three

 endfor
 
 free_lun,one

;Now take coadded images and mosaic them
 result = assemble(mmosnm,sdir+filter+'mosaic'+sls+'mosaics.tmp',scl=scl,zflg=zflg)

;save final results to file
; msz = size(result.mosaic)
 print,'Writing '+sdir+filter+'mosaic'+sls+'final_mosaic.fits'
 moshdr = hdr
 sxaddpar,moshdr,'HISTORY',systime(/utc)+' UTC: mosaic of '+strtrim(npm,2)$
    +' submosaics'
 fits_datmask,/write,sdir+filter+'mosaic'+sls+'final_mosaic.fits',$
    result.mosaic,moshdr,result.master
 offsets = [result.xoff,result.yoff]
 openw,one,sdir+filter+'mosaic'+sls+'final_offsets'+'_f_'+$
    strtrim(n_elements(offsets),2),/get
 writeu,one,offsets
 free_lun,one

;clean up temp files and return final results structure too, just for fun
; rob_fd,sdir+filter+'mosaic'+sls+'*.tmp'
 return,result

end
