FUNCTION lmode, data
   
; Naive script to determine the mode, only take the first 
; in case of double-valued result...
   
   hist = histogram(data) 
   ifin = n_elements(hist)
   xvals = findgen(ifin)+min(data)+0.5
   xvals = xvals(where(hist eq max(hist)))
   return, xvals(0)
   
END 

PRO literstat, data, out, nsigrej=nsigrej,$
               maxiter=maxiter, lower=lower, upper=upper, silent=silent
   
; Program based on dimsum's iterstat:
; Created lam98jan18ucb
; 
; Converted to idl, and made output into a structure.
; 
; procedure iterstat(image)
; 
; # Script to find image statistics excluding deviant pixels
; # 4 August 1992 by John Ward
; # Minor modifications 4 August 1992 MD
; # Various subsequent variations.
; # Latest revision:  18 Aug 1993 MD
;
   
   IF n_params() LE 0 THEN BEGIN 
      print,' '
      print,'literstat, data, out, [nsigrej=], [maxiter=],'
      print,'           [lower=], [upper=], [silent]'
      print,' '
      print,'  N.b. -- The output array ''out'' is a structure:'
      print,'    out.NPIX .MEAN .MEDIAN .SIGMA .MODE .MIN .MAX'
      print,' '
      return 
   ENDIF 
   
   out =  create_struct('Npix',0.,$
                        'Mean',0.,$
                        'Median',0.,$
                        'Sigma',0.,$
                        'Mode',0.,$
                        'min',0.,$
                        'max',0.)
   
   IF keyword_set(nsigrej) EQ 0. THEN nsigrej=5. ; N * sigma for limits
   IF keyword_set(maxiter) EQ 0  THEN maxiter=10 ; Max # of iterations
   IF NOT keyword_set(lower) THEN lower=min(data)
   IF NOT keyword_set(upper) THEN upper=max(data)
   
   data = float(data)           ; Verify that data are floated
   sz = size(data)
   npx = sz(n_elements(sz)-1)   ; Number of relevant pixels
;   lind = lindgen(npx)
   lind = where(data GE lower AND data LE upper)
;;;   data = data(lind)
;    IF n_elements(where(finite(data) EQ 1)) NE n_elements(data) THEN BEGIN 
;       print,'There are NaNs in the array'
;       lind = where(finite(data) EQ 1)
;    ENDIF 
   mn  =  mean  (data(lind))    ; Mean
   sig =  stdev (data(lind))    ; Standard Deviation
   med =  median(data(lind))    ; Median
   mde =  lmode (data(lind))    ; Mode, cf 'lmode' below
   IF NOT keyword_set(silent) THEN print, ' '
   IF NOT keyword_set(silent) THEN print,'    Iter       Npix        Mean         Sigma        Median       Mode'

   Tpx =  0
   m = 1
   WHILE (m LE maxiter AND Tpx NE npx) DO BEGIN 
      npx = Tpx
      ll =  mn - (nsigrej*sig)
      ul =  mn + (nsigrej*sig)
      IF (ll NE min(data(lind)) AND ll LE lower) THEN ll=lower
      IF (ul NE max(data(lind)) AND ul GE upper) THEN ul=upper
      lind = where(data GE ll AND data LE ul)
      IF total(lind) EQ -1 THEN BEGIN 
         print,'Error -- all the pixels masked...'
         return
      ENDIF 
      Tpx =  n_elements(lind)   ; Number of relevant pixels
      mn  =  mean  (data(lind)) ; Mean
      sig =  stdev (data(lind)) ; Standard Deviation
      med =  median(data(lind)) ; Median
      mde =  lmode (data(lind)) ; Mode, cf 'lmode' below
      IF NOT keyword_set(silent) THEN print,m,tpx,mn,sig,med,mde
      m = m+1
   ENDWHILE 
   
   IF NOT keyword_set(silent) THEN print, ' '
   
   out.npix = npx
   out.mean = mn
   out.median = med
   out.sigma = sig
   out.mode = mde
   out.min = ll
   out.max = ul
   
   IF NOT keyword_set(silent) THEN help,/structure,out
   
END 

;+
; NAME:
;       FPULLDOWN
;
; PURPOSE:
;       Fixes the 'pulldown' effect in IRAC images
;
; INPUTS:
;       A 2D image array
;
; KEYWORD PARAMETERS:
;
;       TITLE      - prefix to output fits name; default='fp_'
;       OBJCLIP    - Nsigma threshold for masking objects
;       BOXCARSIZE - Boxcar size for smoothing in creating mask
;       HSIG       - high sigma threshold over projected median
;       LSIG       - low sigma threshold over projected median
;       NOFOWLER   - ignore fowler-based tweak, apply single value 
;       DISPLAY    - switch; display resulting image
;       WRITE      - switch; write out resulting fits image+header
;       HELP       - switch; returns quick keyword summary
;       VERBOSE    - switch; give extra information
;
; OUTPUTS:
;       Returns pulldown-corrected 2D arrays, optionally writing out
;       corrected fits image, and/or displaying to window.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS:
;       Does not work for Channel 4 (yet), returns -1 in that case. 
;
; PROCEDURE:
;       fim = fpulldown(image.fits, TITLE = title, $
;             OBJCLIP = objclip, BOXCARSIZE = boxcarsize,$
;             HSIG = hsig, LSIG = lsig, $
;             /NOFOWLER, /DISPLAY, /WRITE, /HELP, /VERBOSE)
;
; COMMENTS:
;
;       An outline of the algorithm is:
;
;   1. Create an unsharp mask of the image, by boxcar-smoothing the
;   original image by a small kernel
;
;   2. Create a copy of the original image, and replace all 'object'
;   pixels (determined in step 1) by a NAN value
;
;   3. For each column, calculate the object-masked median value,
;   generating a projected median-value row.
;
;   4. Subtract the projected median-averaged row by its own median
;   value, and calculate its statistics.
;
;   5. Using the specified HSIG and LSIG thresholds, identify the
;   deviant columns, and add in the differential correction, possibly
;   with Fowler-based weighting.
;
;   6. Return the corrected image, possibly writing out a new fits
;   image, and/or displaying to screen.
;
; PROCEDURES USED:
;       LITERSTAT
; 
; MODIFICATION HISTORY:
;       LAM '03sep22 - written, with input from DS & MED
;       LAM '03sep23 - modified input method, added Fowler-based
;                      correction of DS
;       LAM '03sep25 - corrected header name identifying Channel
;-
function fpulldown, file, $
                    OBJCLIP = objclip,       $
                    BOXCARSIZE = boxcarsize, $
                    HSIG = hsig,             $
                    LSIG = lsig,             $
                    NOFOWLER = nofowler,     $
                    DISPLAY = display,       $
                    WRITE = write,           $
                    TITLE = title,           $
                    HELP = help,             $
                    VERBOSE = verbose

     if n_elements(help) ne 0 then begin
         print,' '
         print,'fim = fpulldown(''image.fits'', TITLE = title, $'
         print,'      OBJCLIP = objclip, BOXCARSIZE = boxcarsize,$'
         print,'      HSIG = hsig, LSIG = lsig, $'
         print,'      /NOFOWLER, /DISPLAY, /WRITE, /HELP, /VERBOSE)'
         print,' '
         return,-1
     endif

     if n_elements(file) eq 0 then begin
         print,'FPULLDOWN: No fits image provided'
         return,-1
     endif
     if size(file,/type) ne 7 then begin
         print,'FPULLDOWN: Please provide a fits image'
         return,-1
     endif

;     print,'FPULLDOWN: reading '+file
     image=readfits(file,hd,/silent)
;     if fix(sxpar(hd,'ACHANID')) eq 4 then begin
;         print,'FPULLDOWN: This routine is not yet set up for Channel 4'
;         return,-1
;     endif
     
     if n_elements(BOXCARSIZE) eq 0 then BOXCARSIZE = 2.5; 1.5; 5.0
     if n_elements(OBJCLIP)    eq 0 then OBJCLIP = 7.0
     if n_elements(HSIG)       eq 0 then HSIG = 5. ;2.5
     if n_elements(LSIG)       eq 0 then LSIG = 3. ;1.7

;   1. Create an unsharp mask of the image, by boxcar-smoothing the
;   original image by a small kernel

     sz=size(image,/dimen)

;     sim=smooth(image,boxcarsize)
;     sim=median(image,2*boxcarsize) ;RAG, 10-23-03
;     literstat,sim,simst,/silent
;     bad = lonarr(256l*256l) ;RAG, 10-23-03
     robomean,image,2,.2,avg,adev,sdev;,bad=bad ;RAG, 10-23-03
;     id=where(sim gt simst.median+OBJCLIP*simst.sigma)
;     id=where(bad eq 1) ;RAG, 10-23-03
     id=where(image gt avg+objclip*sdev) ;RAG, 10-23-03
     id2=histogram(id/256)
     goodrows = where(id2 le 50,ngr)+min(id/256)
     badrows = where(id2 gt 50,nbr)+min(id/256)
     if ngr eq 0 then begin
      print,'Pulldown correction failed!  No good rows!'
      return,-1
     endif

;   2. Create a copy of the original image, and replace all 'object'
;   pixels (determined in step 1) by a NAN value

     tim=image
; fixed WTR
     if id(0) ne -1 then tim[id]=!values.f_nan
     if nbr gt 0 then tim[*,badrows] = !values.f_nan  ;RAG, 10-23-03
     
;   3. For each column, calculate the object-masked median value,
;   generating a projected median-value row.

     projval=fltarr(sz[0])
     for i=0,sz[0]-1 do projval[i] = median(tim[i,*])

;   4. Subtract the projected median-averaged row by its own median
;   value, and calculate its statistics.

;     medpval = median(projval) 
     pmed = median(projval)
     medpval = median(projval,21) ;RAG, 10-23-03
     medpval[0:9] = medpval[10]
     medpval[246:255]= medpval[245]

     projval = projval - medpval

     bad=long(projval*0)
     robomean,projval,3,.3,pavg,pavgdev,psdev,bad=bad

;     literstat,projval,pst,/silent

;stop

;   5. Using the specified HSIG and LSIG thresholds, identify the
;   deviant columns, and calculate the Fowler-based correction.

;     hthresh = 0.0 + HSIG*pst.sigma
;     lthresh = 0.0 - LSIG*pst.sigma
     hthresh = 0.0 + HSIG*psdev
     lthresh = 0.0 - LSIG*psdev
     fpix = where(projval lt lthresh or projval gt hthresh)
;     projval = projval+medpval-pmed

; default: apply the fowler-weighted correction
     if n_elements(nofowler) eq 0 then begin 
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; n pixels above, m pixels below, fowler sampling FN
; pn = pulldown amount above, pm = pulldown amount below
;
; code calculates what correction pmp is, assuming correction is
; constant over entire column
;
; n*(FN-1)/FN*pm + m*pm = (n+m)*pmp
;
; pm*(n*(FN-1)/FN+m) = (n+m)*pmp
;
;-------------------------------------
;|  pm = (m+n) / (m+(FN-1)/FN*n) * pmp|
;|                                    |
;|  pn = (FN-1)/FN * pm               |
;-------------------------------------
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

         FN = float(sxpar(hd,'AFOWLNUM')) ; Fowler number
         if n_elements(verbose) ne 0 then $
            print,'FPULLDOWN: Fowler number '+strcompress(fix(FN))
;         fpix = where(projval lt lthresh or projval gt hthresh)
         if total(fpix) ne -1 then begin
             timmask=tim
             tim=image
             for i=0,n_elements(fpix)-1 do begin
; the value pmp
                 pmp = projval[fpix[i]]
; the location of the brightest (unmasked) pixel in the original image 
                 brpix = (where(image[fpix[i],*] eq max(image[fpix[i],*])))[0]
; the relevant numbers of pixels above & below brpix from the masked image
                 m = n_elements(where(finite(timmask[fpix[i],0:(brpix-1>0)]) eq 1)) ; below
                 n = n_elements(where(finite(timmask[fpix[i],brpix:(sz[1]-1)]) eq 1)) ; above
                 pm = ( (m+n) / (m+(FN-1)/FN*n) ) * pmp
                 pn = (FN-1)/FN * pm
                 
                 if n_elements(verbose) ne 0 then $
                   print,string(fpix[i] ,format='(i3.3)')+' '+$
                         string(pmp     ,format='(f8.2)')+' '+$
                         string(brpix   ,format='(i3.3)')+' '+$
                         string(m       ,format='(i3.3)')+' '+$
                         string(n       ,format='(i3.3)')+' '+$
                         string(pm      ,format='(f8.2)')+' '+$
                         string(pn      ,format='(f8.2)')+' '+$
                         string( (m*pm+n*pn)/(m+n) ,format='(f8.2)')
                 
                 tim[fpix[i],0:(brpix-1>0)]   = tim[fpix[i],0:(brpix-1>0)]   - pm
                 tim[fpix[i],brpix:(sz[1]-1)] = tim[fpix[i],brpix:(sz[1]-1)] - pn
                 
             endfor
         endif
     endif else begin ; case where not worrying about fowler-based correction
         if n_elements(verbose) ne 0 then print,'FPULLDOWN: Single-value subtraction'
;         fpix = where(projval lt lthresh or projval gt hthresh)
         if total(fpix) ne -1 then begin 
             tim = image
             for i=0,n_elements(fpix)-1 do tim[fpix[i],*]=tim[fpix[i],*]-projval[fpix[i]]
         endif
     endelse
         
;   6. Possibly output and return the corrected image.

     if n_elements(display) ne 0 then begin
         literstat,tim,timst,/silent
         loadct,0
         tvlct,r,g,b,/get
         tvlct,reverse(r),reverse(g),reverse(b)
         plotimage,bytscl(tim,$
                          min=timst.median-   timst.sigma,$
                          max=timst.median+5.*timst.sigma),$
                   /preserve
     endif
         
     if n_elements(write) ne 0 then begin 
         sxaddhist,'pulldown effect corrected',hd
         if n_elements(title) eq 0 then title='c'
    	   p=strpos(file,'bcd_fp',/reverse_search)+6
         outfile=strmid(file,0,p)+$
                 title+$
                 strmid(file,p,strlen(file)-p)
         print,file,' --> ',outfile
         writefits,outfile,tim,hd
     endif
     
     return,tim

 end

